from __future__ import annotations

from tcsoa.gen.AuthorizedDataAccess._2007_06.LicenseManagement import LicenseInput, LicenseTypeAndStatusFilter, LicenseIdsAndTypesResponse, LicenseDetails, GetLicenseDetailsResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class LicenseManagementService(TcService):

    @classmethod
    def removeLicenses(cls, removeLicense: List[LicenseInput]) -> ServiceData:
        """
        This operation removes/detaches ADA Licenses from WorkspaceObject business objects such as Item, ItemRevision,
        Dataset, etc. as specified in each LicenseInput. LicenseInput provides details of the licenses to be removed,
        and the WorkspaceObject business objects to remove the licenses from. Users performing this operation will need
        IP_ADMIN privilege to both the workspace objects and the licenses being removed, if the licenses are of type IP
        or Exclude, while the privilege specified in ADA_license_administration_privilege site preference is needed to
        both the workspace objects and the licenses being removed if the licenses are of ITAR license type. If the user
        does not have necessary privilege to remove any licenses, or if there is any other error while removing
        licenses, the errors are returned as partial errors in ServiceData.
        
        Use cases:
        Use case: 
        You can remove ADA_License business objects from classified WorkspaceObject business objects to revoke access
        for users and/or groups listed on the licenses.
        """
        return cls.execute_soa_method(
            method_name='removeLicenses',
            library='AuthorizedDataAccess',
            service_date='2007_06',
            service_name='LicenseManagement',
            params={'removeLicense': removeLicense},
            response_cls=ServiceData,
        )

    @classmethod
    def setLicenseDetails(cls, licenseInfo: List[LicenseDetails]) -> ServiceData:
        """
        This operation creates or modifies an ADA_License business object for each 'LicenseDetails' supplied. The
        'LicenseDetails' contains information for properties such as license type, license ID, expiry date, reason, and
        associated users and groups, for a given license. If a specified license ID already exists, the rest of the
        property values are updated on that license. However, if the license ID does not exist, then a new license of
        the specified type and ID will be created, and the rest of the properties are set on the created license. The
        user performing the operation will need the privilege specified in the ADA_license_administration_privilege
        site preference to create/modify an ADA License. If a user does not have the necessary privilege or if there is
        a validation error, the operation would fail and the error is returned in the 'ServiceData'.
        
        Use cases:
        Use Case 1: Create an ADA license
        You can create a new ADA license of type ITAR/IP/Exclude using 'setLicenseDetails' operation by providing a
        unique license ID for the license using 'LicenseDetails' structure.
        
        Use Case 2: Modify an ADA License
        The following types of modifications can be done on existing ADA licenses using 'setLicenseDetails' operation.
        Other than the license ID and license type, all other parameters of the ADA licenses can be modified using this
        operation.
        - Set an expiry date on a license
        - Add new users and/or groups to a license
        - Remove users and/or groups from a license
        - Set reason for a license
        
        """
        return cls.execute_soa_method(
            method_name='setLicenseDetails',
            library='AuthorizedDataAccess',
            service_date='2007_06',
            service_name='LicenseManagement',
            params={'licenseInfo': licenseInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteLicense(cls, licenseIds: List[str]) -> ServiceData:
        """
        This operation deletes all ADA_License business objects specified in 'licenseIds' parameter. Users performing
        this operation will need privilege specified in ADA_license_administration_privilege site preference to delete
        the licenses. If the user does not have necessary privilege to delete the licenses, or if any license is
        attached to workspace objects, or, if there is any other error while deleting the licenses, the errors are
        returned as partial errors in 'ServiceData'.
        
        Use cases:
        You can delete ADA_License business objects of type ITAR/IP/Exclude by providing license IDs for the licenses.
        """
        return cls.execute_soa_method(
            method_name='deleteLicense',
            library='AuthorizedDataAccess',
            service_date='2007_06',
            service_name='LicenseManagement',
            params={'licenseIds': licenseIds},
            response_cls=ServiceData,
        )

    @classmethod
    def attachLicenses(cls, attachLicense: List[LicenseInput]) -> ServiceData:
        """
        This operation attaches ADA Licenses to WorkspaceObject business objects such as Item, ItemRevision, Dataset,
        etc. as specified in each 'LicenseInput'. 'LicenseInput' provides details on the licenses to be attached, and
        the WorkspaceObject business objects to attach the licenses to. Users performing this operation will need
        IP_ADMIN privilege to both the workspace objects and the licenses being attached, if the licenses are of type
        IP or Exclude, while ITAR_ADMIN privilege is needed to both the workspace objects and the licenses being
        attached if the licenses are of ITAR license type. If the user does not have necessary privilege to attach any
        licenses, or if there is any other error while attaching licenses, the errors are returned as partial errors in
        'ServiceData'.
        
        Use cases:
        You can attach ADA_License business objects to classified WorkspaceObject business objects to grant access for
        users and/or groups listed on the licenses.
        """
        return cls.execute_soa_method(
            method_name='attachLicenses',
            library='AuthorizedDataAccess',
            service_date='2007_06',
            service_name='LicenseManagement',
            params={'attachLicense': attachLicense},
            response_cls=ServiceData,
        )

    @classmethod
    def getLicenseDetails(cls, licenseIds: List[str]) -> GetLicenseDetailsResponse:
        """
        This operation gets the properties for each ADA_License business object specified in 'licenseIds' and returns
        them in 'LicenseDetails' structures as part of 'GetLicenseDetailsResponse'. The 'LicenseDetails' contains
        information for properties such as license type, license ID,  expiry date, reason, and associated users and
        groups, for a given license. If there is no matching license for a given license ID, 'LicenseDetails' structure
        for that license will contain 'Not_found' as the value for license ID, while the rest of the parameters will
        contain NULL values. However, if the user does not have READ privilege to any license or if there is any
        unexpected error while getting property information, the errors are returned in the 'ServiceData' of
        'GetLicenseDetailsResponse'.
        
        Use cases:
        
        Use Case 1: Get properties of an ADA License
        You can get properties of an ADA_License business object of type ITAR/IP/Exclude using 'getLicenseDetails'
        operation by providing license ID for the license.
        
        Use Case 2: Check the existence of an ADA License
        You can check if an ADA_License businesses object with given license ID exists using 'getLicenseDetails'
        operation.
        """
        return cls.execute_soa_method(
            method_name='getLicenseDetails',
            library='AuthorizedDataAccess',
            service_date='2007_06',
            service_name='LicenseManagement',
            params={'licenseIds': licenseIds},
            response_cls=GetLicenseDetailsResponse,
        )

    @classmethod
    def getLicenseIdsAndTypes(cls, licenseFilterInput: List[LicenseTypeAndStatusFilter]) -> LicenseIdsAndTypesResponse:
        """
        This operation gets all the ADA_License business objects based on license type and license status specified in
        'licenseFilterInput' parameter. Licenses of a specific type can be queried for by specifying the license type
        as 'ITAR_License',' IP_License' or 'Exclude_License'. A value of  'ALL' for the license type returns all types
        of ADA licenses. If the license status specified is 'ALL', all ADA licenses are returned, else only unlocked
        and unexpired licenses are returned. For the ADA_License business objects found, details like license IDs and
        license types are returned in 'LicenseIdAndType' structures as part of 'LicenseIdsAndTypesResponse'. If there
        is any unexpected error while getting licenses, the errors are returned in the 'serviceData' of
        'LicenseIdsAndTypesResponse'.
        
        Use cases:
        Use Case 1: Get ADA Licenses based on type and status
        You can get ADA_License business objects using 'getLicenseIdsAndTypes' operation based on specified license
        type and license status.
        """
        return cls.execute_soa_method(
            method_name='getLicenseIdsAndTypes',
            library='AuthorizedDataAccess',
            service_date='2007_06',
            service_name='LicenseManagement',
            params={'licenseFilterInput': licenseFilterInput},
            response_cls=LicenseIdsAndTypesResponse,
        )



from __future__ import annotations

from tcsoa.gen.EditContext._2016_04.DataManagement import ChangeContextMap, SetChangeContextResponse
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def setChangeContext(cls, inputs: ChangeContextMap) -> SetChangeContextResponse:
        """
        This operation will configure business objects so that the objects can be edited or inspected in a context
        without changing the public versions of the objects. This operation can also be used to remove the
        configuration of business objects in order to edit or inspect the public versions of the objects.
        This operation does not have any effect on business objects that are not minor revisable and such objects will
        always be returned without being configured.
        The operation can be used to configure multiple sets of objects using different contexts.
        
        Use cases:
        Use Case 1: User wants to edit properties of a set of business objects in a context with the intention to 
        collaborate with other users.
        - User creates a Fnd0EditContext object that abstracts the context within which the edit needs to be performed.
        - The Fnd0EditContext object is provided as input to the operation along with the business objects that need to
        be edited.
        - The configured business objects are returned in the response and can be used for performing edits in the
        context.
        
        
        
        Use Case 2: User wants to edit properties of a set of business objects in a context provided by a ChangeNotice.
        - User creates a ChangeNotice object that abstracts the context within which the edit needs to be performed.
        - The ChangeNotice object is provided as input to the operation along with the business objects that need to be
        configured.
        - The configured business objects are returned in the response and can be used for performing edits in the
        context.
        
        
        
        Use Case 3: User wants to query a property belonging to the public version of a business object that is
        currently configured for editing in a collaborative space.
        - User passes a null value for the context along with the business object that needs to be inspected in public.
        - The unconfigured business objects are returned in the response and any queries performed on the objects will
        return public data.
        
        """
        return cls.execute_soa_method(
            method_name='setChangeContext',
            library='EditContext',
            service_date='2016_04',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SetChangeContextResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanFile
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReplaceFileInput(TcBaseObj):
    """
    This structure contains the input to the 'replaceFiles' operation.  This includes the transient volume write ticket
    used to upload the new file.  This is the FMS write ticket issued by the 'getTransientFileTicketsForUpload'
    operation.  The 'replaceFiles' operation uses this ticket to locate the data file in the transient volume.  This
    structure also includes a flag to indicate if the original file name should be updated.
    
    :var imanFile: The ImanFile object whose file is to be replaced.
    :var newFileTicket: The transient file write ticket that was used to upload the new file to the transient volume.
    :var retainOriginalFileName: If set to true, the original file name of the ImanFile object will not be changed.  If
    false, the original file name will be changed to the file name of the new file as set in 'newFileTicket'.
    """
    imanFile: ImanFile = None
    newFileTicket: str = ''
    retainOriginalFileName: bool = False



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AttributeDetailElement(TcBaseObj):
    """
    Comparison result of an attribute in an Attribute Group.
    
    :var attributeNames: Corresponding attribute names for each member 	of the equivalentObjects vector. The size of
    this vector matches the size of equivalentObjects vector and it has corresponding indices.       
    :var isDifferent: False if the property value is equal in the source and target, otherwise true.
    """
    attributeNames: List[str] = ()
    isDifferent: bool = False


@dataclass
class AttributeGroupAndFormComparisonResponse(TcBaseObj):
    """
    For each supplied attribute group the operation returns the list of its attributes, the attributes' values for each
    supplied source and target, and the result of comparing each attribute on all supplied sources and targets.
    
    :var serviceData: Contains the property values for each property in each attribute group for each supplied
    equivalent set and any partial errors.
    :var attributeGroupAndFormDetails: List of attribute groups information elements - one for each input equivalent
    set.
    """
    serviceData: ServiceData = None
    attributeGroupAndFormDetails: List[AttributeGroupAndFormDetail] = ()


@dataclass
class AttributeGroupAndFormDetail(TcBaseObj):
    """
    Attribute groups details of an equivalent set of objects. 
    
    :var index: Index of equivalent set in the input vector for which these details were calculated.
    :var equivalentObjects: The list of all equivalent business objects in the input equivalent set (all equivalent
    sources in sequence and then all targets in sequence).
    :var attributeGroupDetailElements: Attribute groups details of this equivalent set. Each element in the list
    represents one attribute group.
    """
    index: int = 0
    equivalentObjects: List[BusinessObject] = ()
    attributeGroupDetailElements: List[AttributeGroupsDetailElement] = ()


@dataclass
class AttributeGroupsDetailElement(TcBaseObj):
    """
    Comparison results of an Attribute Group.
    
    :var attributeGroupName: The name of this attribute group.
    :var isDifferent: True if any of the properties in this attribute group are different, otherwise false.
    :var attrGroupsAndForms: Corresponding attribute groups and forms objects for each member of the equivalentObjects
    vector. 		The size of this vector matches the size of equivalentObjects vector and it has corresponding indices.
    :var attributeDetails: The list of details for each mapped property in the attribute group.
    """
    attributeGroupName: str = ''
    isDifferent: bool = False
    attrGroupsAndForms: List[BusinessObject] = ()
    attributeDetails: List[AttributeDetailElement] = ()


@dataclass
class BusinessObjectVec(TcBaseObj):
    """
    Vector of business objects.
    
    :var businessObjects: Vector of business objects.
    """
    businessObjects: List[BusinessObject] = ()


@dataclass
class ConnectedObjectsComparisonResponse(TcBaseObj):
    """
    For each input set of equivalent objects a vector of comparison results of their connected elements (can be full
    match, partial match, or multiple match) and for each object in the set a detailed connected elements breakdown.
    
    :var serviceData: Object that captures any partial errors.
    :var connectedObjectsDetails: List of connected objects information elements - one for each input equivalent set.
    """
    serviceData: ServiceData = None
    connectedObjectsDetails: List[ConnectedObjectsDetail] = ()


@dataclass
class ConnectedObjectsDetail(TcBaseObj):
    """
    Comparison results of connected elements of an equivalent set of objects (can be full match, partial match, or
    multiple match) and for each object in the set a detailed connected elements breakdown. 
    
    :var index: The index of equivalent set in the input list for which these details were calculated.
    :var matchResults: A list of the comparison results (0 means full match, 1 means partial match, 2 means multiple
    match) of all rows of the output table. Each vector element is the result of comparing one equivalent group of
    connected objects (one row in the table).
    :var connectedObjectsDetails: A list of input equivalent object and its detailed set of connected elements. This
    vector contains the whole ouput results table, where each vector element represents a column in the table.
    """
    index: int = 0
    matchResults: List[int] = ()
    connectedObjectsDetails: List[ConnectedObjectsDetailElement] = ()


@dataclass
class ConnectedObjectsDetailElement(TcBaseObj):
    """
    Contains connected objects of the equivalentObject.
    
    :var equivalentObject: The input equivalent object for which these details are defined.
    :var connectedObjects: The set of connected objects for this input object. The outer vector is 	insync with the
    index of the matchResults, each element representing one cell in the output table, and the inner vector will be the
    group of connected elements that are equivalent to each other (if there are no equivalent elements in this group at
    all the inner vector will be empty). Each inner list will have only one element, and all equivalent elements will
    be put in separate inner list.
    """
    equivalentObject: BusinessObject = None
    connectedObjects: List[BusinessObjectVec] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BOMEdit
from tcsoa.gen.ChangeManagement._2008_06.ChangeManagement import FindContextDataInputs, GetAllChangeHomeFolderResponse, CreateChangeItemInputs, GetBOMEditResponse, UpdateSupercedureData, CreateBOMEditInput, CreateChangeItemsResponse, FindContextDataResponse, FindSupersedureResponse, ConfigureChangeSearchData, GetBOMEditInput, CreateSupercedureInput, CreateBOMEditResponse, CreateSupercedureResponse, UpdateChangeProperties
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ChangeManagementService(TcService):

    @classmethod
    def createSupercedures(cls, supercedureProperties: List[CreateSupercedureInput]) -> CreateSupercedureResponse:
        """
        This operation creates Supercedure objects for each 'CreateSupercedureInput' supplied. A Supercedure represents
        a relation that graphically displays the deleted components and the new components that replace them. Transfer
        Supercedure is a specific type that is created where a component is transferred from one assembly to another
        level in a different assembly.
        
        Use cases:
        An assembly modification results in the components being added to or removed from the assembly. Supercedure is
        a component replacement where a user specifically marks one or more added components with one or more removals.
        This use case is accomplished by the 'createSupercedure' operation. The Supercedure is created from:
        - The original product structure 
        - The new product structure which is currently being edited in response to change.
        
        """
        return cls.execute_soa_method(
            method_name='createSupercedures',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'supercedureProperties': supercedureProperties},
            response_cls=CreateSupercedureResponse,
        )

    @classmethod
    def deleteSupercedures(cls, supercedureTobeDeleted: List[BusinessObject]) -> ServiceData:
        """
        This operation deletes the Supercedures specified as the input parameter. A Supercedure defines the
        relationship between any number of additions or deletions implemented within a Bill of Material (BOM) assembly.
        The 'deleteSupercedures' enables a user to remove a Supercedure or delete a BOM add/cancel from a Supercedure.
        
        Use cases:
        A user can delete the supercedures by calling the 'deleteSupercedures' which takes in the list of Supercedure
        objects as input parameter.
        """
        return cls.execute_soa_method(
            method_name='deleteSupercedures',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'supercedureTobeDeleted': supercedureTobeDeleted},
            response_cls=ServiceData,
        )

    @classmethod
    def findContextData(cls, findcontextRequest: List[FindContextDataInputs]) -> FindContextDataResponse:
        """
        This operation provides a flexible way of retrieving objects or data for a variety of contexts to satisfy the
        RichClient usability requirements for BOM Change, Supersedure, and Genealogy in Teamcenter Structure Manager. 
        There are helper functions in the RichClient to facilitate the consumption and conversion of the retrieved
        information.  In other words, this operation may pose challenges to users of this operation who are unfamiliar
        with the intended usage of the returned details.  For RichClient developers, it is better to use the helper
        functions instead.
        The context is specified by the context string 'contextRelName' of an input structure, and based on the
        specified context, different input arguments can be supplied via a change revision object reference, and/or a
        pair of generic business object references.  The objects or data returned will be captured in an output
        structure.  The following are the possible values for the context string:
        - CM_impacted_of_solution
        - CM_associated_change
        - CM_supersedure_of_bomedit
        - CM_bomedits_of_occ
        - CM_supersedures_of_solution
        - CM_get_supersedures_of_bomcancel
        - CM_get_supersedures_of_bomadd
        - CM_get_first_supersedure
        - CM_get_pure_adds_cancels
        - CM_get_supersedure_for_WU
        - CM_get_pureAddsCancels_for_WU
        - CM_get_change_of_supersedure
        - CM_get_initial_rendering_details
        - CM_refresh_rendering_without_supersedure
        - CM_refresh_rendering_with_supersedure
        - CM_get_bomline_adds
        - CM_get_bomline_cancels
        
        
        
        Use cases:
        Use Case 1: Finding the impacted item given a change revision and a solution item
        For the input structure 'FindContextDataInputs',use CM_impacted_of_solution for 'contextRelName', supply the
        change revision as 'changeRev' and the solution item as 'primaryContextInputData'.  The requested impacted item
        can be found in the object list 'contextOutputData' of the corresponding output structure
        'FindContextDataOutput'.
        Use Case 2: Finding the associated change revision given a solution item
        For the input structure 'FindContextDataInputs',use CM_associated_change for 'contextRelName', supply the
        solution item as 'primaryContextInputData'.  The requested associated change revision can be found in the
        'changeRev' of the corresponding output structure 'FindContextDataOutput'.
        Use Case 3: Finding the supersedure given a BOMEdit object
        For the input structure 'FindContextDataInputs',use CM_supersedure_of_bomedit for 'contextRelName', supply the
        BOMEdit object as 'primaryContextInputData'.  The requested supersedure can be found in the object list of the
        corresponding output structure 'FindContextDataOutput'.
        Use Case 4: Finding all the BOMEdit objects associated with a given Occurrence
        For the input structure 'FindContextDataInputs',use CM_bomedits_of_occ for 'contextRelName', supply the
        occurrence object as 'primaryContextInputData'.  The requested BOMEdit object can be found in the object list
        of the corresponding output structure 'FindContextDataOutput'.
        Use Case 5: Finding all supersedures created for a given solution item
        For the input structure 'FindContextDataInputs',use CM_supersedures_of_solution for 'contextRelName', supply
        the solution item as 'primaryContextInputData'.  The requested supersedure objects can be found in the object
        list 'contextOutputData' of the corresponding output structure 'FindContextDataOutput'.
        Use Case 6: Finding all the supersedures associated with a given BOMEdit of type cancel
        For the input structure 'FindContextDataInputs',use CM_get_supersedures_of_bomcancel for 'contextRelName',
        supply the BOMEdit object as 'primaryContextInputData'.  The requested supersedure objects can be found in the
        object list 'contextOutputData' of the corresponding output structure 'FindContextDataOutput'.
        Use Case 7: Finding all the supersedures associated with a given BOMEdit of type add
        For the input structure 'FindContextDataInputs',use CM_get_supersedures_of_bomadd for 'contextRelName', supply
        the BOMEdit object as 'primaryContextInputData'.  The requested supersedure objects can be found in the object
        list 'contextOutputData' of the corresponding output structure 'FindContextDataOutput'.
        Use Case 8: Finding the first supersedure given a change object and a BOM line
        For the input structure 'FindContextDataInputs',use CM_get_first_supersedure for contextRelName, supply the
        change revision as changeRev and the BOM line as 'primaryContextInputData'.  The requested first supersedure
        can be found in the object list 'contextOutputData' of the corresponding output structure
        'FindContextDataOutput'.
        Use Case 9: Finding the Pure Adds and Pure Cancels for a given BOM line
        For the input structure 'FindContextDataInputs',use CM_get_pure_adds_cancels for 'contextRelName', supply the
        BOM line as 'primaryContextInputData'.  The requested Pure Adds and Pure Cancels can be found in the object
        list 'contextOutputData' of the corresponding output structure 'FindContextDataOutput'.  The list of
        vBomEditCount can be used to sort out the Adds and the Cancels.
        Use Case 10: Finding the first supersedure for a given bom line in the context of a whereused assembly
        For the input structure 'FindContextDataInputs',use CM_get_supersedure_for_WU for 'contextRelName', supply the
        BOM line as 'primaryContextInputData' and the whereused assembly as 'secondaryContextInputData'.  The requested
        first supersedure can be found in the object list 'contextOutputData' of the corresponding output structure
        'FindContextDataOutput'.
        Use Case 11: Finding all the Pure Adds and Pure Cancels for a given bom line in the context of a whereused
        assembly
        For the input structure 'FindContextDataInputs',use CM_get_pureAddsCancels_for_WU for 'contextRelName', supply
        the BOM line as 'primaryContextInputData' and the whereused assembly as 'secondaryContextInputData'.  The
        requested Pure Adds and Pure Cancels can be found in the object list 'contextOutputData' of the corresponding
        output structure 'FindContextDataOutput'.  The list of vBomEditCount can be used to sort out the Adds and the
        Cancels.
        Use Case 12: Finding the associated change revision given a supersedure
        For the input structure 'FindContextDataInputs',use CM_get_change_of_supersedure for 'contextRelName', supply
        the supersedure as 'primaryContextInputData'.  The requested associated change revision can be found in the
        'changeRev' of the corresponding output structure 'FindContextDataOutput'.
        Use Case 13: Finding all the BOM lines and the supersedures given a change revision and a pair of BOM windows
        representing the solution and the impacted items.
        For the input structure 'FindContextDataInputs',use CM_get_initial_rendering_details for 'contextRelName',
        supply the change revision as 'changeRev', the BOM windlow for the solution item as 'primaryContextInputData'
        and the BOM windlow for the impacted item as 'secondaryContextInputData'.  The requested BOM lines can be found
        in the object list 'contextOutputData' of the corresponding output structure 'FindContextDataOutput'.
        Use Case 14: Finding all the solution BOM lines and the impacted BOM lines given a change revision and an Add
        line and a Cancel line without including the supersedure data
        For the input structure 'FindContextDataInputs',use CM_refresh_rendering_without_supersedure for
        contextRelName, supply the change revision as 'changeRev', the Add line as 'primaryContextInputData' and the
        Cancel line as 'secondaryContextInputData'.  The requested BOM lines can be found in the object list of the
        corresponding output structure.
        Use Case 15: Finding all the solution BOM lines and the impacted BOM lines given a change revision and an Add
        line and a Cancel line including the supersedure data
        For the input structure 'FindContextDataInputs',use CM_refresh_rendering_with_supersedure for 'contextRelName',
        supply the change revision as 'changeRev', the Add line as 'primaryContextInputData' and the Cancel line as
        'secondaryContextInputData'.  The requested BOM lines can be found in the object list of the corresponding
        output structure. 
        Use Case 16: Finding all the Add BOM lines for a given supersedure
        For the input structure 'FindContextDataInputs',use CM_get_bomline_adds for 'contextRelName', supply the
        supersedure as 'primaryContextInputData'.  The requested Add BOM lines can be found in the object list
        'contextOutputData' of the corresponding output structure.
        Use Case 17: Finding all the Cancel BOM lines for a given supersedure
        For the input structure 'FindContextDataInputs',use CM_get_bomline_cancels for 'contextRelName', supply the
        supersedure as 'primaryContextInputData'.  The requested Cancel BOM lines can be found in the object list of
        the corresponding output structure.
        
        
        """
        return cls.execute_soa_method(
            method_name='findContextData',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'findcontextRequest': findcontextRequest},
            response_cls=FindContextDataResponse,
        )

    @classmethod
    def findSupersedure(cls, bomedit: List[BOMEdit]) -> FindSupersedureResponse:
        """
        This operation finds the Supersedures based on the BOMEdits supplied as input. A BOM change which encompasses a
        BOM Add or a BOM delete enables a Supersedure to be created. A user can create a Supersedure which defines the
        additions and deletions within a BOM.
        
        Use cases:
        Use case 1: Create Supercedures
        An assembly modification results in the components being added to or removed from the assembly. Supercedure is
        a component replacement where a user specifically marks one or more added components with one or more removals.
        The Supercedure is created from:
        - The original product structure 
        - The new product structure which is currently being edited in response to change.
        
        
        Use case 2: Find Supersedures
        Finding a BOM Supersedure can be accomplished by calling the 'findSupersedure' for each input 'bomeditRequest'.
        """
        return cls.execute_soa_method(
            method_name='findSupersedure',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'bomedit': bomedit},
            response_cls=FindSupersedureResponse,
        )

    @classmethod
    def getAllChangeHomeFolders(cls) -> GetAllChangeHomeFolderResponse:
        """
        This operation returns a list of all Change Home folders.  This operation is created to support the Change Home
        UI in Change Manager.  Change Home allows users to define their own set of folders whose contents are
        determined by the searches/queries that are executed when the folders are expanded.  When Change Home is being
        opened, this operation is used to retrieve the folders that are defined for use by the current user.  Change
        Home will then display them based on their settings.
        
        Use cases:
        Retreiving a list of all Change Home folders for the user
        Where there is a need to retrieve all the Change Home folders for the current user session, this operation can
        be invoked via an instance of the 'ChangeManagementService'.  No parameters are required.  The response of the
        operation is captured in the form of 'GetAllChangeHomeFolderResponse'.  It is important to note that the
        folders retrieved will include BOTH those defined at the site level as well as those defined at the user level.
         Site level folders are available to all users, whereas user level folders are only available to the current
        user.  The caller program can make intelligent use of the list by inspecting the details of each individual
        folder that are captured within the structure 'ConfigureChangeSearchData'.
        """
        return cls.execute_soa_method(
            method_name='getAllChangeHomeFolders',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={},
            response_cls=GetAllChangeHomeFolderResponse,
        )

    @classmethod
    def getBOMEdits(cls, bomeditRequest: List[GetBOMEditInput]) -> GetBOMEditResponse:
        """
        This operation is to support the BOM Change tab in Change Manager which displays the details of the BOM Change
        associated with the selected change revision in the Open Change View.
        
        Use cases:
        Finding all BOMEdit objects for a given change revision
        When there is a need to find all BOMEdit objects of a certain type for a given change revision, this operation
        can be invoked via an instance of the 'ChangeManagementService'.  The caller program needs to be aware of all
        the possible types (as represented by 'bomEditType' in the 'GetBOMEditInput' structure) of BOMEdit in order to
        process the returned objects correctly.  It is important to note that a 'PSBOMViewRevision' object for the
        solution item will need to be located and supplied as the 'affectedBvr' element within the 'GetBOMEditInput'
        structure.  Any BOMEdit objects found will be captured in the list of 'GetBOMEditOutput' structures of the
        'GetBOMEditResponse'.
        """
        return cls.execute_soa_method(
            method_name='getBOMEdits',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'bomeditRequest': bomeditRequest},
            response_cls=GetBOMEditResponse,
        )

    @classmethod
    def updateChangeItems(cls, updateProps: List[UpdateChangeProperties]) -> ServiceData:
        """
        Updates a vector of change items by using the inputs provided in the argument
        returns the updated objects via the services data.
        """
        return cls.execute_soa_method(
            method_name='updateChangeItems',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'updateProps': updateProps},
            response_cls=ServiceData,
        )

    @classmethod
    def updateSupercedures(cls, supercedureTobeUpdated: List[UpdateSupercedureData]) -> ServiceData:
        """
        This operation updates Supercedures for each 'UpdateSupercedureData' supplied. The 'UpdateSupercedureData'
        contains information for properties such as bomAddorCancelFlag, operation, tags and the supTag which specifies
        the Supercedure business object to be updated. A Supercedure represents a relation that graphically displays
        the deleted components and new components that replace them. Once a Supercedure is created, new BOM elements
        can be added or removed which allows the user to indicate the Supercedure to be edited and updated.
        
        Use cases:
        Use case1: Create Supercedures
        An assembly modification results in the components being added to or removed from the assembly. Supercedure is
        a component replacement where a user specifically marks one or more added components with one or more removals.
        This use case is accomplished by the operation createSupercedure with the input structure
        'CreateSupercedureInput' utilizing a list of Supercedure properties. The Supercedure is created from:
        - The original product structure 
        - The new product structure which is currently being edited in response to change.
        
        
        Use case2: Update Supercedures
        An existing Supercedure can be updated at point in time and this is achieved by the operation
        'updateSupercedures'. This operation checks if the operation is of type ADD or CANCEL and correspondingly adds
        or deletes a list of business objects from the Supercedure business object specified in the input parameter.
        The input structure elements utilize the Change Management BOM_ADD/BOM_CANCEL and
        OPERARTION_TYPE_ADD/OPERATION_TYPE_CANCEL constants in the SOA based on which a decision is made by the user to
        append or remove components from the assembly.
        """
        return cls.execute_soa_method(
            method_name='updateSupercedures',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'supercedureTobeUpdated': supercedureTobeUpdated},
            response_cls=ServiceData,
        )

    @classmethod
    def configureChangeSearches(cls, input: List[ConfigureChangeSearchData]) -> ServiceData:
        """
        Add or remove a change specific searches folder with the given inputs.  This operation is created to support
        the Manage Saved Search capability for Change Home within Change Manager.  It is important to note that, if
        called directly, there is a possibility of overwriting any pre existing configurations for Change Home, at site
        and/or user level.  So, USE IT WITH CAUTION.
        
        Use cases:
        Adding and Removing change home folder(s)
        This can be accomplished by calling the 'configureChangeSearches' operation with a list of
        'ConfigureChangeSearchData' structures which specify a number of necessary attributes of the folders to be
        created or removed.  Each 'ConfigureChangeSearchData' structure will contain the name of the folder (which
        should be unique), a visible/invisible indicator, the saved search/query associated with the folder, any query
        criteria and their corresponding values, a site/user level indicator, and an add/remove operation indicator.  A
        site level folder is available to all site users, whereas a user level folder is available only to the user of
        the current user session.  The add/remove operation indicator will determine if the operation is to add or to
        remove the folder.
        """
        return cls.execute_soa_method(
            method_name='configureChangeSearches',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def createBOMEdits(cls, bomEditRequest: List[CreateBOMEditInput]) -> CreateBOMEditResponse:
        """
        This operation is to support the creation of BOM Change in Structure Manager.  A number of BOMEdit objects are
        created when the solution item BOMViewRevision (BVR) is saved and then compared to the impacted item BVR in two
        side by side BOM windows.  Central to this operation is the change revision object that ties all the BOMEdit
        objects together to represent the whole BOM Change.
        
        Use cases:
        Creating a BOM Change within the Structure Manager
        A change revision has been created with an identified impacted item.  A user working on a solution will first
        revise the impacted item, relate the new revision to the change revision as solution item, and then make
        structural changes to the new revision in Structure Manager. When the work is complete, the solution item is
        saved.   The save operation will detect if the object saved is associated with a change revision as the
        solution item.  If it is, it will proceed to render the solution item and the impacted item in two side by side
        BOM windows, and then invoke the 'createBOMEdits' operation via an instance of the 'ChangeManagementService'
        with the 'CreateBOMEditInput' structure that contains the associated change revision and the two BOM windows .
        """
        return cls.execute_soa_method(
            method_name='createBOMEdits',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'bomEditRequest': bomEditRequest},
            response_cls=CreateBOMEditResponse,
        )

    @classmethod
    def createChangeItems(cls, changeItemProps: List[CreateChangeItemInputs]) -> CreateChangeItemsResponse:
        """
        Creates an list of Change Items of specified type and any associated data with each change item including
        Change Item Revision,
        Change Item Master Form, and Change Item Revision Master form based on the input attribute structures.
        Also it creates relationship between the objects passed as in input properties and assigns created change item
        to a specified
        user as requestor, analyst, admin or role that is specified in the input.
        It also updates any additional properties of the change item that is passed in the input.
        If the relation data contains a change item an attempt will be made to propagate values from this parent change
        item to
        the newly create item based on the propagation rules defined in the change configuration also an attempt will
        be made to
        create the prpoer types of relations between the change item that is created and input objects.
        Incase of Problem Report (PR) if the CMRoles is empty the requestor of the PR will be the same as the creator
        of the PR.
        """
        return cls.execute_soa_method(
            method_name='createChangeItems',
            library='ChangeManagement',
            service_date='2008_06',
            service_name='ChangeManagement',
            params={'changeItemProps': changeItemProps},
            response_cls=CreateChangeItemsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Classification._2017_05.Classification import GetClassDefinitionsResponseNX
from typing import List
from tcsoa.base import TcService


class ClassificationService(TcService):

    @classmethod
    def getClassDefinitionsNX(cls, classIDs: List[str]) -> GetClassDefinitionsResponseNX:
        """
        This operation returns Classification class definition, its attribute definitions including the details of
        associated Classification KeyLOV (stxt) objects.
        In comparison to 'getClassDefinitons' it contains additionally the NX Unit ID for each Classification attribute.
        
        Use cases:
        User wants to get details of classification class along with all associated class attributes. This operation is
        combination of getClassDescriptions and getAttributesForClasses2, but provides information in a different
        format to cater additional information like extended properties, attribute dependency, KeyLOVs (stxt) and also
        the NX Unit ID associated to class attributes.
        """
        return cls.execute_soa_method(
            method_name='getClassDefinitionsNX',
            library='Internal-Classification',
            service_date='2017_05',
            service_name='Classification',
            params={'classIDs': classIDs},
            response_cls=GetClassDefinitionsResponseNX,
        )



from __future__ import annotations

from tcsoa.gen.Internal.GlobalMultiSite._2010_02.LowlevelOwnershipTransfer import ObjectsForOwnershipTransferResponse, UpdateOwnershipTransferResponse, TransferOwnershipResponse
from tcsoa.base import TcService


class LowlevelOwnershipTransferService(TcService):

    @classmethod
    def transferOwnership(cls, tcGSMessageId: str, dryrun: bool, isSrcSiteExtinct: bool, fmsTicketOfObjs: str) -> TransferOwnershipResponse:
        """
        This is an internal operation which transfers ownership of objects that are in the input file. This is executed
        at the target site. Ownership is changed for more than one object. It is advisable to run this operation in dry
        run mode first and then perform actual ownership transfer. In dry run mode, actual ownership is not changed but
        validation is performed for the list of input objects.
        
        Output of this operation is a file which has status of ownership change. Format of the file in dry run mode
        with source extinct flag is as follows
        #Dry Run Report
        # TIE transfer data status (low level) 
        # Syslog FileName    :  C:\Temp\tcxml_xfer_ownership10302d08.syslog
        #$SourceSiteID: 100001
        #$TargetSiteID: 100002
        #$DryRun: Yes
        #$SourceExtinct: Yes
        #PUID        IslandAnchorUIDs    Status
        ArD53SsdAABaaA ArD53SsdAABaaA 0
        QLK53SsdAABaaA ArD53SsdAABaaA 0
        
        #$Dry Run With Extinct
        #$Report of objects at target site db that are 
        #$owned by source site and not in consolidation list 
        QLK53SsdAABaBA    Item
        
        Following are the status codes and their interpretation
        0 : Success
        1: Failure [Island is inconsistent]
        2: Object is missing at target site
        3: New object at target site
        
        
        Use cases:
        If getObjectsForOwnershipTransfer operation succeeds, then this operation is invoked by the BPEL process of
        Global Services when user runs sitcons_xfer_owner_mgr utility of site consolidation.
        """
        return cls.execute_soa_method(
            method_name='transferOwnership',
            library='Internal-GlobalMultiSite',
            service_date='2010_02',
            service_name='LowlevelOwnershipTransfer',
            params={'tcGSMessageId': tcGSMessageId, 'dryrun': dryrun, 'isSrcSiteExtinct': isSrcSiteExtinct, 'fmsTicketOfObjs': fmsTicketOfObjs},
            response_cls=TransferOwnershipResponse,
        )

    @classmethod
    def updateOwnershipTransfer(cls, tcGSMessageId: str, changeOwnershipToSite: int, dryrun: bool, isSrcSiteExtinct: bool, fmsTicketOfObjs: str) -> UpdateOwnershipTransferResponse:
        """
        This is an internal operation. The input file has information about the status of failed or successful islands.
        It generates a report which has status of ownership transfer. Report is saved in the database as TEXT dataset.
        In dry run mode, failed islands are marked as inconsistent. If dry run mode is false then function does
        following:
            (1) Updates the ownership transfer status to the accountability table
            (2) Updates the ownership at the source site
        
        
        Use cases:
        This is invoked by the BPEL process of Global Services when user runs sitcons_xfer_owner_mgr utility of site
        consolidation.
        """
        return cls.execute_soa_method(
            method_name='updateOwnershipTransfer',
            library='Internal-GlobalMultiSite',
            service_date='2010_02',
            service_name='LowlevelOwnershipTransfer',
            params={'tcGSMessageId': tcGSMessageId, 'changeOwnershipToSite': changeOwnershipToSite, 'dryrun': dryrun, 'isSrcSiteExtinct': isSrcSiteExtinct, 'fmsTicketOfObjs': fmsTicketOfObjs},
            response_cls=UpdateOwnershipTransferResponse,
        )

    @classmethod
    def getObjectsForOwnershipTransfer(cls, tcGSMessageId: str, changeOwnershipToSite: int, dryrun: bool) -> ObjectsForOwnershipTransferResponse:
        """
        This is an internal operation which collects source objects that are replicated to a target site (specified in
        changeOwnershipToSite) using site consolidation tool and generates a report. This is executed on the exporting
        site. For this operation, the dry run mode does not have much significance. The report has the following format
        #Accountability Data
        # puids for ownership transfer (low level)
        # Syslog-FileName    :  C:\Temp\tcxml_xfer_ownership26585f2e.syslog
        #$DryRun: Yes
        #$SourceSiteID: 100001
        #$TargetSiteID: 100002
        #PUID         IslandAnchorUIDs
        QLK53SsdAABaaA ArD53SsdAABaaA
        QLG53SsdAABaaA ArD53SsdAABaaA
        ArN53SsdAABaaA ArD53SsdAABaaA
        
        
        Use cases:
        It gets invoked by BPEL process of Global services when user runs sitcons_xfer_owner_mgr utility of site
        consolidation.
        """
        return cls.execute_soa_method(
            method_name='getObjectsForOwnershipTransfer',
            library='Internal-GlobalMultiSite',
            service_date='2010_02',
            service_name='LowlevelOwnershipTransfer',
            params={'tcGSMessageId': tcGSMessageId, 'changeOwnershipToSite': changeOwnershipToSite, 'dryrun': dryrun},
            response_cls=ObjectsForOwnershipTransferResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetViewModelForCreateInfo(TcBaseObj):
    """
    GetViewModelForCreateInfo represent the parameters required to load ViewModelObject and render 'Grid:Row' view in
    editable state, to author WorkspaceObject of given input businessObjectType.
    
    :var businessObjectType: Internal type name of the object to be created.
    :var propertyNames: A list of internal property names displayed in table, for which mapped properties of
    BusinessObject to be determined.
    :var parent: The parent object of type Awb0Element under which child node information to be retrieved. Supported
    types are: Awb0DesignElement a child type of Awb0Element.
    """
    businessObjectType: str = ''
    propertyNames: List[str] = ()
    parent: BusinessObject = None


@dataclass
class GetViewModelForCreateResponse(TcBaseObj):
    """
    GetViewModelForCreateResponse contains information required to create WorkspaceObject.
    
    :var viewModelCreateInObjsJsonStrings: The UI definition to render an editable row. Json strings that contains the
    ViewModelObject structures which provide information for the CreateInput object, returned as a part of ServiceData
    plain objects. This is like the viewModelObjectJsonString used in getDeclarativeStyleSheet2 and
    loadViewModelForEditing2 operation.
    :var columnPropToCreateInPropMap: A map (string, string) of property names on column and CreateInput property name
    pairs  of the input businessObjectType.
    :var createHtmlProviders: A list of html panel defined input businessObjectType create stylesheet definition.
    :var serviceData: ServiceData containing partial error if any.
    """
    viewModelCreateInObjsJsonStrings: List[str] = ()
    columnPropToCreateInPropMap: StringMap = None
    createHtmlProviders: List[str] = ()
    serviceData: ServiceData = None


"""
A map of displayed property names on column and corresponding CreateInput property name pairs (string, string).
"""
StringMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Manufacturing._2014_06.DataManagement import EstablishOriginLinkInfo, CloneAssemblyInputData, CloneAssemblyResponse, AddOrRemoveContextsInfo, EstablishOriginLinkResponse, ProcResourceResponseInfo
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2014_06.ResourceManagement import CheckToolParametersResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Manufacturing._2014_06.StructureSearch import SrchConnectedLinesResponse
from tcsoa.base import TcService


class ResourceManagementService(TcService):

    @classmethod
    def checkToolParameters(cls, icoIds: List[str], checkLevel: str, checkTypes: List[str]) -> CheckToolParametersResponse:
        """
        This operation checks for a list of tools if they define the required attribute values to create their 3D
        graphics or use them as cutters in NX CAM.  The list of tools is identified by their internal classification
        object IDs (ICO IDs).   The caller can select the level and type of checking that gets performed. The operation
        will return a check result consisting of a status and report for each tool  being checked.
        """
        return cls.execute_soa_method(
            method_name='checkToolParameters',
            library='Manufacturing',
            service_date='2014_06',
            service_name='ResourceManagement',
            params={'icoIds': icoIds, 'checkLevel': checkLevel, 'checkTypes': checkTypes},
            response_cls=CheckToolParametersResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def cloneAssemblyAndProcessObjects(cls, input: List[CloneAssemblyInputData]) -> CloneAssemblyResponse:
        """
        This operation clones an assembly in manufacturing Bill of Material (mBOM) as a reaction to changes in
        engineering BOM (eBOM) planning or clones a process in Bill Of Processes as a reaction to changes in mBOM
        planning. As a result of this operation, the original assembly/process is cloned and occurrence effectivity is
        set correctly on both original and cloned assembly/process.
        
        For eBOM-mBOM cloning some of the parts from the new assembly in the mBOM are replaced with the new parts that
        were introduced to the eBOM as a replacement in that unit effectivity.
        
        For mBOM-BOP cloning the process that used to refer to the original assembly in the mBOM is cloned and all
        references are fixed to point at the new assembly.
        
        Use cases:
        Clone subassembly with effectivity Use Case.
        Use Case Description:
        In this use case, the user detects changes in the eBOM, and would like to make the appropriate changes to the
        Mbom. The changes entail cloning the target assembly where the parts are in the mBom.
        Use Case Goal:
        Handle changes in eBOM by creating the appropriate stock-able assembly in the mBom.
        
        Description:
        This use case may be regarded as a direct continuation of the Clone assembly with effectivity Use Case. In this
        use case, the user continues to handle the change, by cloning the process to which the "original" assembly was
        assigned (and its hierarchy), and fixing all internal references to the new copy.
        Use Case Goals:
        Handle changes in mBom by creating the appropriate structure to consume it in the BOP.
        
        """
        return cls.execute_soa_method(
            method_name='cloneAssemblyAndProcessObjects',
            library='Manufacturing',
            service_date='2014_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CloneAssemblyResponse,
        )

    @classmethod
    def addOrRemoveAssociatedContexts(cls, input: List[AddOrRemoveContextsInfo]) -> ServiceData:
        """
        Generic operation to set or remove different kinds of associations. 
        The input parameter to the operation governs the target context to associate to, list of source contexts, the
        relation name to associate or disassociate with and the additional action required to add or remove the context.
        
        
        Use cases:
        - Use Case 1: Add the origin link between the Generic Bill Of Processes (BOP) and Product BOP or Product BOP
        and Plant BOP.
        
        
                Description: An origin link (association) can be added between Generic BOP and Product BOP or Product
        BOP and Plant BOP.
        
        - Use Case 2: Remove the origin link between the Generic BOP and Product BOP or Product BOP and Plant BOP.
        
        
                Description: An origin link can be removed between Generic BOP and Product BOP or Product BOP and Plant
        BOP. 
        
        - Use Case 3: Remove the origin link and allocations between BOPs
        
        
                Description: After removing the origin link between Generic BOP and Product BOP or Product BOP and
        Plant BOP, the user will have an option to remove the origin link between the allocations. Also the user will
        have option to remove the allocated  operations or processes. 
        """
        return cls.execute_soa_method(
            method_name='addOrRemoveAssociatedContexts',
            library='Manufacturing',
            service_date='2014_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def establishOriginLink(cls, input: EstablishOriginLinkInfo) -> EstablishOriginLinkResponse:
        """
        This service operation establishes the origin link between process and/or operations. It can directly establish
        origin link between the input source (process and/or operation from Generic Bill Of Process (BOP) or Product
        BOP and target(s) (process and/or operation from Product BOP or Plant BOP. 
        The input parameter to the operation governs whether origin link need to be established or it's simply a dry
        run, whether hierarchy should be considered and the criteria based on which the origin link is established.
        
        Use cases:
        - Use case 1 : Establish origin link between Process/Operation.
        
        
        Description : An origin link can be established between Process/Operation where source is a process/operation
        from Generic BOP or Product BOP and targets are the processes/operations from Product BOP or Plant BOP. The
        input parameter "action" must be specified as "Link".
        
        - Use case 2 : Dry-run without establishing the origin link.
        
        
        Description : This service can identify the candidate targets to be linked to the process/operation from
        Generic BOP or Product BOP to the processes/operations from Product BOP or Plant BOP on the basis of the
        provided criteria. The input parameter "action" must be specified as "DryRun".
        """
        return cls.execute_soa_method(
            method_name='establishOriginLink',
            library='Manufacturing',
            service_date='2014_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=EstablishOriginLinkResponse,
        )

    @classmethod
    def getProcessResourceRelatedInfo(cls, input: List[BusinessObject], objectPreference: str) -> ProcResourceResponseInfo:
        """
        This service operation fetches the information related to process resource (work content performed by
        resource(s) in a manufacturing plant ) and the operation/process that are allocated/unallocated to
        process-resource. The operation takes object for which information is required. The object can be process
        station, process line, process area, or process resource. The object must be in the context of Plant Bill of
        Processes (BOP).
        
        Use cases:
        Use Case 1: Fetching process resource for a given process area.
        This operation can be used to fetch the process resource of specific process area. In context of a Plant BOP,
        this operation can be called on the process area BOM line. In response, all the process lines and process
        stations under the process area are fetched apart from  process resources for each process station and
        allocated/unallocated operations/processes. The result is populated on a view providing information on process
        resources, process station it belongs to and allocated/unallocated operations/processes.
        
        Use Case 2: Fetching processes/operations for a given process resource.
        This operation can be used to fetch the process/operation that are allocated to process resource. 
        In context of a Plant BOP, this operation can be called on the process resource BOM line. In response, all
        allocated/unallocated operations/processes are fetched. The result is populated on a view providing information
        on process resources and allocated operations/processes.
        
        Use Case 3: Fetching processes/operaions that are not allocated to process resource for a given process area 
        and process
        This operation can be used to fetch the process/operation that are not allocated to process resource. In
        context of a Plant BOP, this operation can be called on the process area or process BOM line. In response, all
        unallocated operations/processes are fetched. The result is populated on a view providing information on
        process resources and unallocated operations/processes.
        """
        return cls.execute_soa_method(
            method_name='getProcessResourceRelatedInfo',
            library='Manufacturing',
            service_date='2014_06',
            service_name='DataManagement',
            params={'input': input, 'objectPreference': objectPreference},
            response_cls=ProcResourceResponseInfo,
        )


class StructureSearchService(TcService):

    @classmethod
    def searchConnectedLines(cls, inputConnLines: List[BusinessObject]) -> SrchConnectedLinesResponse:
        """
        This operation searches and returns the connected BOMLine objects for the input connection BOMLine objects.
        """
        return cls.execute_soa_method(
            method_name='searchConnectedLines',
            library='Manufacturing',
            service_date='2014_06',
            service_name='StructureSearch',
            params={'inputConnLines': inputConnLines},
            response_cls=SrchConnectedLinesResponse,
        )



from __future__ import annotations

from tcsoa.gen.OfficeOnline._2017_11.OfficeOnline import LaunchInfoOutputResponse, LaunchInfoInput
from typing import List
from tcsoa.base import TcService


class OfficeOnlineService(TcService):

    @classmethod
    def getLaunchInfo(cls, inputs: List[LaunchInfoInput]) -> LaunchInfoOutputResponse:
        """
        This operation retrieves the launch information for a Teamcenter Dataset that a client can use to access Office
        Online to view or edit a Microsoft document. The launch URL is the address to use to connect to the Office
        Online Server (OOS) to open a Microsoft Office file in Office Online.
        
        Integration with Office Online requires implementation of the Web Application Open Platform Interface (WOPI)
        protocol.  For this protocol, the OOS is called the WOPI client and the Teamcenter component that OOS
        communications with is called the WOPI host.  The Teamcenter WOPI host implements REST based APIs specific to
        the WOPI protocol for the Office Online WOPI client to use to work with files stored in Teamcenter.
        
        Use cases:
        A Teamcenter client finds and selects a Dataset for a Microsoft Office document and chooses to view or edit the
        document.  The client calls this operation to retrieve the launch URL to access Office Online to view or edit
        the Microsoft Office document.
        """
        return cls.execute_soa_method(
            method_name='getLaunchInfo',
            library='OfficeOnline',
            service_date='2017_11',
            service_name='OfficeOnline',
            params={'inputs': inputs},
            response_cls=LaunchInfoOutputResponse,
        )



from __future__ import annotations

from tcsoa.gen.Common import ObjectPropertyPolicy
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class RegisterIndex(TcBaseObj):
    """
    Contains the index to be used for unregistering.
    
    :var registryIndex: Index to be used for unregistering.
    """
    registryIndex: int = 0


@dataclass
class SetPolicyResponse(TcBaseObj):
    """
    The policy ID and full definition of the object property policy.
    
    :var policyId: Unique ID for this object property policy.
    :var policy: The full definition of the object property policy.
    """
    policyId: str = ''
    policy: ObjectPropertyPolicy = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetAMImpactedObjectsResponse(TcBaseObj):
    """
    Data structure that holds the UIDs of Teamcenter business objects whose READ access is impacted by the Access
    Manager rule tree changes, a Boolean flag that indicates if full reindex is needed or not and the service data with
    any partial errors that might happen while finding the impacted objects.
    
    :var impactedUids: List of unique identifiers (UIDs) of Teamcenter business objects whose READ access is impacted
    by the Access Manager rule changes.
    :var queriedDateTime: The date and time at which the AM impacted objects are queried for. Require this to keep
    track of sync case.
    :var serviceData: The Service Data.
    """
    impactedUids: List[str] = ()
    queriedDateTime: datetime = None
    serviceData: ServiceData = None


@dataclass
class GetIndexedObjectsResponse(TcBaseObj):
    """
    Structure containing list of indexed object UIDs and service data with partial errors.
    
    :var indexedObjectsUIDs: The list of indexed object UIDs.
    :var serviceData: The Service Data.
    """
    indexedObjectsUIDs: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class GetModifiedObjectsToSyncResponse(TcBaseObj):
    """
    Structure containing the list of modified object UIDs since the last sync run and the timestamp when the query is
    executed to find the modified objects.
    
    
    
    :var modifiedObjectsUIDs: The list of object UIDs that were modified after they been exported and indexed.
    :var queriedDateTime: The timestamp at which the modified objects are queried.
    :var serviceData: The Service Data.
    """
    modifiedObjectsUIDs: List[str] = ()
    queriedDateTime: datetime = None
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, EPMTask
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class WorkflowService(TcService):

    @classmethod
    def performAction2(cls, task: EPMTask, action: str, comments: str, password: str, supportingValue: str, supportingObject: BusinessObject) -> ServiceData:
        """
        This operation performs an action on a workflow task. The following actions are supported: 
        - Assign
        - Start
        - Complete
        - Skip
        - Suspend
        - Resume
        - Undo
        - Perform
        - Approve
        - Reject
        - Promote
        - Demote
        - Claim
        
        
        
        Use cases:
        User can perform a workflow task from worklist folder in rich or thin client. Similarly, workflow tasks can be
        performed or signed-off using office client as well. This operation can be used to perform or sign-off the
        workflow tasks.
        """
        return cls.execute_soa_method(
            method_name='performAction2',
            library='Workflow',
            service_date='2012_10',
            service_name='Workflow',
            params={'task': task, 'action': action, 'comments': comments, 'password': password, 'supportingValue': supportingValue, 'supportingObject': supportingObject},
            response_cls=ServiceData,
        )



from tcsoa.gen.Participant._2018_11.services import ParticipantService as imp0
from tcsoa.base import TcService


class ParticipantService(TcService):
    addParticipants = imp0.addParticipants
    reassignParticipants = imp0.reassignParticipants
    removeParticipants = imp0.removeParticipants



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class DerivativeRuleResponse(TcBaseObj):
    """
    DerivativeRuleResponse represents the output of the published execute Derive CAE Structure operation.
    
    The root of the output structure is returned as a part of created object in the Service Data. A log is also
    returned with the results of the derivative rule applied to the input structure and the output item created. Any
    failures in creation of the output item or relationships are also returned as a part of the activity log.
    
    The following partial errors may be returned:
    
    - 206845 - Error occurred during Derive operation.
    - 206845 - Derive  Engine unable to load/read/parse XMLRendering.
    - 206643 - CAE_dataMapping_file preference not defined.
    - 206647 - Item creation failed, operation aborted.
    - 206648 - Occurrence creation failed, operation aborted.
    - 206649 - Unknown attribute found.
    - 206650 - Object not modifiable, set attribute operation failed.
    - 206651 - Form creation failed.
    - 206652 - BOMView creation failed.
    - 206653 - Unable to save the Item in the Newstuff folder.
    - 206664 - Error in relationship creation.
    - 206665 - Item node line definition missing in Data Map.
    
    
    
    :var serviceData: The root of the output structure is returned as a part of Created Object in the Service Data.
    :var activityLog: The text of activityLog contains the Item ID and revision of the root of created output structure
    . In case of failure of derive operation, the text will report failure. This text is used for intermediate
    reporting when executeDeriveCAEStructure is called in a loop for multiple structure creation from rich client. User
    is  informed of the status of ongoing operation by displaying this text in a string viewer. If failure is reported,
    user can choose to terminate the entire operation from rich client. The details of failure can be then retrieved
    from ServiceData.
    """
    serviceData: ServiceData = None
    activityLog: str = ''



from tcsoa.gen.AuthorizedDataAccess._2007_06.services import LicenseManagementService as imp0
from tcsoa.gen.AuthorizedDataAccess._2009_10.services import LicenseManagementService as imp1
from tcsoa.gen.AuthorizedDataAccess._2017_05.services import LicenseManagementService as imp2
from tcsoa.gen.AuthorizedDataAccess._2013_05.services import LicenseManagementService as imp3
from tcsoa.gen.AuthorizedDataAccess._2018_06.services import LicenseManagementService as imp4
from tcsoa.gen.AuthorizedDataAccess._2012_09.services import LicenseManagementService as imp5
from tcsoa.base import TcService


class LicenseManagementService(TcService):
    attachLicenses = imp0.attachLicenses
    attachLicenses2 = imp1.attachLicenses
    attachOrDetachLicensesFromObjects = imp2.attachOrDetachLicensesFromObjects
    createOrUpdateLicense = imp3.createOrUpdateLicense
    createOrUpdateLicense2 = imp4.createOrUpdateLicense
    deleteLicense = imp0.deleteLicense
    getLicenseDetails = imp0.getLicenseDetails
    getLicenseDetails2 = imp1.getLicenseDetails2
    getLicenseDetails3 = imp5.getLicenseDetails3
    getLicenseDetails4 = imp3.getLicenseDetails4
    getLicenseIdsAndTypes = imp0.getLicenseIdsAndTypes
    removeLicenses = imp0.removeLicenses
    setLicenseDetails = imp0.setLicenseDetails
    setLicenseDetails2 = imp1.setLicenseDetails
    setLicenseDetails3 = imp5.setLicenseDetails



from tcsoa.gen.Internal_Mmv._2012_09.services import SpatialStructureManagementService as imp0
from tcsoa.base import TcService


class SpatialStructureManagementService(TcService):
    acquireSpatialHierarchy = imp0.acquireSpatialHierarchy
    createMmvCursor = imp0.createMmvCursor
    getNodeBBox = imp0.getNodeBBox
    getSpatialCellsReadTickets = imp0.getSpatialCellsReadTickets
    isSpatialHierarchyLatest = imp0.isSpatialHierarchyLatest
    releaseMmvCursor = imp0.releaseMmvCursor
    releaseSpatialHierarchy = imp0.releaseSpatialHierarchy



from __future__ import annotations

from typing import List
from tcsoa.gen.Multisite._2007_06.ImportExport import RemoteImportInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ImportExportService(TcService):

    @classmethod
    def remoteImport(cls, infos: List[RemoteImportInfo]) -> ServiceData:
        """
        The 'remoteImport' operation is used to import given business objects from the target site to the source site.
        The import options supplied in the 'RemoteImportInfo' structure influences the business object import.
        
        E.g. In case user supplies 'Include All revision' as TRUE, from target site, source site will import all
        versions of the dataset.  
        
        When option(s) are not specified in the RemoteImportInfo structure, by default user session preferences at the
        importing site are considered for import.  Default values are mentioned under the 'RIAttributeInfo' structure.
        
        
        
        Use cases:
        - In the Multi-Site federation user can share the business objects among the participating sites, the client
        shares business object(s) using the remote import or remote export functionality. Incase of the remote import
        the source site need to publish the item to the object directory services. 
        - To remote import the business object, first the source site user creates an Item and publishes it to the
        default ODS. The target site user performs remote search with the item id and imports it to the target site
        using the remote import operation. On successful completion of remote import operation, the item is imported to
        the target site database.
        - Also remote import of business objects can be achieved using the command line utility data_share
        f=remote_import. 
        
        
        
        Exceptions:
        >.
        """
        return cls.execute_soa_method(
            method_name='remoteImport',
            library='Multisite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'infos': infos},
            response_cls=ServiceData,
        )



from tcsoa.gen.EditContext._2015_07.services import DataManagementService as imp0
from tcsoa.gen.EditContext._2016_04.services import DataManagementService as imp1
from tcsoa.gen.EditContext._2016_10.services import DataManagementService as imp2
from tcsoa.gen.EditContext._2014_12.services import DataManagementService as imp3
from tcsoa.base import TcService


class DataManagementService(TcService):
    getReferencerEditContexts = imp0.getReferencerEditContexts
    setChangeContext = imp1.setChangeContext
    setChangeContext2 = imp2.setChangeContext2
    setEditContext = imp3.setEditContext



from __future__ import annotations

from tcsoa.gen.Internal.ProjectManagementAw._2019_12.ScheduleManagementAw import ProgramViewInfo, ManageProgramViewResponse
from tcsoa.base import TcService


class ScheduleManagementAwService(TcService):

    @classmethod
    def manageProgramView(cls, programViewInfo: ProgramViewInfo) -> ManageProgramViewResponse:
        """
        Manages a ProgramView object based on the information provided in the ProgramViewInfo. This operation can
        create, configure and save a ProgramView object. You can also load the ProgramView by filtering and grouping
        ScheduleTask objects based on the ProgramView configuration. The ProgramView configuration contains the list of
        Schedule UIDs, filters and groups.
        """
        return cls.execute_soa_method(
            method_name='manageProgramView',
            library='Internal-ProjectManagementAw',
            service_date='2019_12',
            service_name='ScheduleManagementAw',
            params={'programViewInfo': programViewInfo},
            response_cls=ManageProgramViewResponse,
        )



from tcsoa.gen.Visualization._2011_02.services import DataManagementService as imp0
from tcsoa.gen.Visualization._2013_05.services import DataManagementService as imp1
from tcsoa.gen.Visualization._2011_02.services import StructureManagementService as imp2
from tcsoa.gen.Visualization._2013_05.services import StructureManagementService as imp3
from tcsoa.gen.Visualization._2013_12.services import StructureManagementService as imp4
from tcsoa.gen.Visualization._2016_03.services import DataManagementService as imp5
from tcsoa.base import TcService


class DataManagementService(TcService):
    createLaunchFile = imp0.createLaunchFile
    createLaunchInfo = imp1.createLaunchInfo
    getMetaDataStampWithContext = imp5.getMetaDataStampWithContext


class StructureManagementService(TcService):
    createVisSC = imp2.createVisSC
    createVisSCsFromBOMs = imp2.createVisSCsFromBOMs
    createVisSCsFromBOMs2 = imp3.createVisSCsFromBOMs
    createVisSCsFromBOMsWithOptions = imp4.createVisSCsFromBOMsWithOptions



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class MaturityData(TcBaseObj):
    """
    Structure containing the result of maturity check and other relevant information
    
    :var evaluatedObject: The BusinessObject on which rules are evaluated. For example, a BOMLine from the structure is
    a valid object on which certain rules are evaluated.
    :var ruleResults: Map containing the result of the evaluation. The key of the map is uid of the rule and value of
    the map is list of PropertyData containing the result of the evaluation along with relevant data.
    
    The valid propertyName members of the PropertyData is:
    
    - status:  Specifies the status of the evaluation. Valid values of status are "Pass", "Fail", "Partial" and "NA"
    (Not Applicable).  "Pass" signifies that rule on evaluatedObject is passed, "Fail" signifies failed, "Partial"
    signifies partially passed and "NA" signifies that the rule is not applicable for the evaluated object.
    - statusDisplayValue: Display value of the rule evaluated on the object. It may not be same as outcome of rule.
    - ruleUID: uid of the rule
    
    
    :var objectLevelStatus: Overall result and status of evaluated object. 
    
    The valid propertyName member of the PropertyData is:
    
    - status:  Specifies the status of the evaluation. Valid values of status are "Pass", "Fail" and "Partial".  "Pass"
    signifies that all rules on evaluatedObject passed, "Fail" signifies that all rules failed and "Partial" signifies
    that some rule passed. Rules that are not applicable for the evaluated object are ignored for determining the
    overall status of the object.
    
    """
    evaluatedObject: BusinessObject = None
    ruleResults: CollectiveData = None
    objectLevelStatus: List[PropertyData] = ()


@dataclass
class MaturityReportRequest(TcBaseObj):
    """
    The input object to service. It contains below parameters - 
    
    inputObjects -    The input contains the BOMLine which acts as a scope for maturity check. The objects in the
    hierarchy of the scope are candidates for the check.
    
    ruleList -    The list of rules for which maturity is to be evaluated. Some of the rules may not be applicable to
    all objects in the scope.
    
    supportingInformation -    Additional supporting information that could be used to evaluate the maturity. Currently
    this input is not used by the operation.
    
    
    :var inputObjects: A list of BOMLine for which the maturity report is required. All objects in the hierarchy of the
    line are evaluated for the maturity check.
    :var selectedRules: A list of rules to evaluate the maturity. The 
    key and value (string, list(PropertyData)) of the map are uid of the rule and corresponding data respectively. The
    latter is list of structure PropertyData which has property name of the rule and data of that property
    respectively. The valid property name is "ruleUID" with value as UID of the rule.
     
    The rule in the list may be applicable to specific object type. The operation fetches those objects for a given
    scope and evaluates the rule. Both evaluated objects and status of the evaluation is reported in the response.
    
    For any two rules from the list, one may evaluate an object type different than that of another.
    :var supportingInformation: Additional supporting information required to evaluate the maturity. 
    Currently this input is not used by operation.
    """
    inputObjects: List[BusinessObject] = ()
    selectedRules: CollectiveData = None
    supportingInformation: List[PropertyData] = ()


@dataclass
class MaturityReportResponse(TcBaseObj):
    """
    The response is  the map containing maturity data and the service data. The map contains the evaluated object and
    its maturity status along with additional information related to checks. The evaluated objects type may not be of
    same type and may not be applicable to all the rules specified in input of the operation.
    
    :var serviceData: The service data containing partial errors.
    :var maturityResult: The result of getMaturityData operation, contains BusinessObjects and their maturity data.
    """
    serviceData: ServiceData = None
    maturityResult: MaturityResult = None


@dataclass
class PropertyData(TcBaseObj):
    """
    The detail information of the data in the form of property and its value.
    
    :var propertyName: Key to identify the data in the collection.
    :var value: Value of the property.
    :var dataType: Type of the data. It can be "boolean", "character", "date", "float", "double", "integer", "string"
    and "businessObject".
    """
    propertyName: str = ''
    value: List[str] = ()
    dataType: str = ''


"""
The result of getMaturityData operation, contains BusinessObjects and their maturity data.
"""
MaturityResult = Dict[BusinessObject, MaturityData]


"""
A list of rules to evaluate the maturity. The 
key and value (string, list(PropertyData)) of the map are uid of the rule and corresponding data respectively. The latter is vector of structure PropertyData which has property name of the rule and data of that property respectively. The valid property name is "ruleUID" with value as UID of the rule.
 
The rule in the list may be applicable to specific object type. The operation fetches those objects for a given scope and evaluates the rule. Both evaluated objects and status of the evaluation is reported in the response.

For any two rules from the list, one may evaluate an object type different than that of another.
"""
CollectiveData = Dict[str, List[PropertyData]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Requirementsmanagement._2011_06.RequirementsManagement import PublishColumnConfigInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class RequirementsManagementService(TcService):

    @classmethod
    def publishColumnConfiguration(cls, input: List[PublishColumnConfigInfo]) -> ServiceData:
        """
        This operation publishes the Column Configuration that is Mark as Publishable by the user. This operation will
        get the column configuration values from the specified Fnd0ColumnConfiguration object and create the site
        preferences. After creating the site preference it will set the "IsPublished" property to true present on the
        specified Fnd0ColumnConfiguration object.  So by creating the site preference these column configuration will
        be visible to all present users in the system. This operation converts the user preferences to the site level
        preferences in Teamcenter Context so that all users can use it. When user save the column configuration,
        applied on the BOM structure then it will store all applied column configuration values as user protection
        scope preferences. After that user can mark the same column configuration as Mark as Publishable that means
        user want that column configuration to be available to other users in the system. So Administrator privileged
        user can publish the column configuration so it will be available to all other users in the system. The user
        which originally saved the column configuration will see two preferences with same name, one with protection
        scope user and other one with site protection scope.
        
        Use cases:
        You can publish the column configuration that is Mark as Publishable by the user so that it will be visible to
        all other users and others users can apply the publish column configuration on BOM structure.
        """
        return cls.execute_soa_method(
            method_name='publishColumnConfiguration',
            library='Requirementsmanagement',
            service_date='2011_06',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Cad._2008_03.StructureManagement import AskChildPathBOMLinesResponse, AskChildPathBOMLinesInfo
from tcsoa.gen.Cad._2007_12.DataManagement import CreateOrUpdatePartsPref
from tcsoa.gen.Cad._2007_01.DataManagement import CreateOrUpdatePartsResponse
from typing import List
from tcsoa.gen.Cad._2008_03.DataManagement import PartInfo3, ResolveAttrMappingsResponse, ResolveAttrMappingsInfo
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def resolveAttrMappings(cls, info: List[ResolveAttrMappingsInfo]) -> ResolveAttrMappingsResponse:
        """
        Retrieves CAD attribute mapped properties for item revisions or datasets.  Attribute Mapping is a scheme
        whereby Teamcenter attributes can be retrieved or set via a defined path to the attribute from the starting
        object, usually a dataset.  For example, a mapped attribute can be defined in the client integration with a
        particular name ATTR1.  Using Teamcenter attribute mapping, the customizer can define a path named ATTR1 from a
        starting point item revision type or dataset type to the actual attribute that holds the value for ATTR1. The
        client integration then can access the attribute using the attribute mapping title ATTR1 and the starting point
        object.
        For more information about Attribute Mapping including examples with syntax, please consult the "Configuring
        attribute mapping section" of the Application Administration Guide in Teamcenter Online Help.
        
        Use cases:
        A user performs a File Open operation on an existing Teamcenter dataset.  The client integration has defined an
        attribute mapping in Teamcenter for that Dataset type.  The resolveAttrMappings call performed as a part of the
        File Open, sends the mapping definition, defined by the mapping title, and the Dataset as input.  The operation
        traverses from the input Dataset to the mapped property which in this case resides on the Datasets parent Item
        Revision.  The operation will return the item revision and the mapped property name such that the client
        integration can retrieve the property value from the item revision.  The value is then displayed in the
        attribute data for the dataset in the client integration.
        """
        return cls.execute_soa_method(
            method_name='resolveAttrMappings',
            library='Cad',
            service_date='2008_03',
            service_name='DataManagement',
            params={'info': info},
            response_cls=ResolveAttrMappingsResponse,
        )

    @classmethod
    def createOrUpdateParts(cls, info: List[PartInfo3], pref: CreateOrUpdatePartsPref) -> CreateOrUpdatePartsResponse:
        """
        CreateOrUpdateParts allows the user to create or update a set of parts using Items, Item Revisions, Datasets
        and ExtraObjects and also changes the ownership of the newly created object to the user/group supplied.
        If the user supplies a valid Item object reference without specifying a valid item revision object reference or
        id, then only the item will be updated.
        If the user specifies a valid item object reference with a null item revision object reference and a non-null
        revision id, then a new item revision will be created and attached to the item with no relations from the new
        item revision to the previous item revision.
        If the user specifies a valid item object reference and a valid item revision object reference, then the item
        and item revision and related objects will be updated.
        If no item object reference or item revision object references are specified then a new item and item revision
        and related objects will be created.
        All objects created and updated will be returned in the ServiceData.
        All partial errors will contain the clientIDs for all items related to the error message, i.e. if a dataset
        encounters an error, then the ID for that erro will be the item client ID concantentated with the item revision
        id contantenated with the dataset client ID, all separated by semi-colons ( ; ).
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateParts',
            library='Cad',
            service_date='2008_03',
            service_name='DataManagement',
            params={'info': info, 'pref': pref},
            response_cls=CreateOrUpdatePartsResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def askChildPathBOMLines(cls, input: List[AskChildPathBOMLinesInfo]) -> AskChildPathBOMLinesResponse:
        """
        Given one or more sets of product structure information containing child paths specified by PS Occurrence
        Thread chains, this method returns the corresponding BOM Lines.
        """
        return cls.execute_soa_method(
            method_name='askChildPathBOMLines',
            library='Cad',
            service_date='2008_03',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=AskChildPathBOMLinesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ContainsRenderableFilesResponse(TcBaseObj):
    """
    Contains Renderable Files Response structure consisting of a list of renderable files and the service data.
    
    :var objectsWithRenderableFiles: A list of objects (from 'selectedObjects') that contain renderable files.
    :var serviceData: Standard return; includes any error information.
    """
    objectsWithRenderableFiles: List[BusinessObject] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GenerateContextIDsInput(TcBaseObj):
    """
    This structure is the input sructure for the generateContextSpecificIDs service.
    This contains the informanion of context name and number of IDs to be generated for that context name.
    
    :var clientID: A unique string supplied by the caller. This ID is used to identify returned
    GenerateContextIDResponse elements and Partial Errors associated with this input GenerateContextIDsInput.
    :var contextName: Name of the context for which IDs to be generated. A context name is a string that can be up to
    256 characters long. This string should not be empty, in order to generate IDs, an error is returned if user tries
    to generate ID for a empty context name.
    :var numberOfIDs: Represents number of IDs to be generated for a given context name. This is a mandatory field and
    should not be 0 or any negative number to generate IDs. An error is returned, if user tries to generate ID for a
    context name with invalid value (0 or negative number) for this field.
    """
    clientID: str = ''
    contextName: str = ''
    numberOfIDs: int = 0


@dataclass
class GenerateContextSpecificIDsResponse(TcBaseObj):
    """
    Contains the returned information of generateContextSpecificIDs service.
    
    :var serviceData: Partial Errors are returned in the ServiceData when service generateContextSpecificIDs fails to
    generate IDs of any of the context name. Errors are mapped back to input context name based on the client ID field
    provided in structure GenerateContextIDsInput.
    :var generatedContextIDValues: A list of GeneratedContextIDs, which contains each of the context names and IDs
    generated for it by service generateContextSpecificIDs.
    """
    serviceData: ServiceData = None
    generatedContextIDValues: List[GeneratedContextIDs] = ()


@dataclass
class GeneratedContextIDs(TcBaseObj):
    """
    Contains the combination of context name and its generated IDs based on the user input.
    
    :var contextName: Name of the context for which IDs has been generated. This value is provided by the user in
    GenerateContextIDsInput for generating IDs.
    :var generatedIDs: List of generated IDs for a given context name. This contains one or more generated IDs for a
    given context name based on the user input in input structure GenerateContextIDsInput.
    """
    contextName: str = ''
    generatedIDs: List[str] = ()


@dataclass
class PropData(TcBaseObj):
    """
    This structure holds information about Teamcenter business object, its last saved date when exported to client and
    list of property name/value pair information.
    
    :var businessObject: The business object for which the properties to be set. All business object types are
    supported.
    :var vecPropNameVal: List of property name/value pair information.
    :var lastSavedDateExportedToClient: Last saved date of the object when object was exported to client.
    """
    businessObject: BusinessObject = None
    vecPropNameVal: List[PropertyNameValuesStruct] = ()
    lastSavedDateExportedToClient: datetime = None


@dataclass
class PropertyNameValuesStruct(TcBaseObj):
    """
    This structure contains property name, list of old values for the property and list of new values to be set for the
    property.
    
    :var propertyName: Name of the property
    :var oldValues: A vector of old values of the property for comparison with the latest database values to indentify
    any overwrite before saving. The calling client is responsible for converting the different property types (int,
    float, date etc.) to a string using the appropriate toXXXstring functions in the SOA client framerwork&rsquo;s
    Property class.
    :var newValues: A vector of new values of the property to save. The calling client is responsible for converting
    the different property types (int, float, date etc.) to a string using the appropriate toXXXstring functions in the
    SOA client framerwork&rsquo;s Property class.
    """
    propertyName: str = ''
    oldValues: List[str] = ()
    newValues: List[str] = ()


@dataclass
class SetPropsAndDetectOverwriteResponse(TcBaseObj):
    """
    Response structure for setPropertiesAndDetectOverwrite operation. It returns the information about overwritten
    objects and the properties/objects which are not saved due to conflict.
    
    :var data: This is the service data. It contains the updated objects and their properties.
    :var overwriteDetectedObjPropMap: The  map ( business object / list of string ) contains the objects and properties
    list for which overwrite condition has been detected.
    """
    data: ServiceData = None
    overwriteDetectedObjPropMap: ObjectPropertiesMap = None


"""
This map has information about object and its properties.
"""
ObjectPropertiesMap = Dict[BusinessObject, List[str]]


"""
This map has information about the name of the options and its values.
"""
OptionsMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Administration._2015_07.UserManagement import CreateOrUpdateUserInputs
from tcsoa.gen.BusinessObjects import User, Group, Role
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExistingUserDetailsForGroupMember(TcBaseObj):
    """
    A structure of User object and its attributes.
    
    :var user: Existing User to be added. This User will be added to the Group with the specified Role.
    :var userAttributes: List of userAttributes associated with each User.
    """
    user: User = None
    userAttributes: UserAttributesStructure = None


@dataclass
class NewUserDetailsForGroupMember(TcBaseObj):
    """
    A structure of new User to be created with its attributes.
    
    :var user: A list of User properties . Using these properties a new User will be created and added to the Group
    with specified Role.
    :var userAttributes: Additional GroupMember proeprties for the new User.
    """
    user: CreateOrUpdateUserInputs = None
    userAttributes: UserAttributesStructure = None


@dataclass
class UserAttributesStructure(TcBaseObj):
    """
    A structure of User attributes.
    
    :var isGroupAdmin: If true, user will be the group administrator; otherwise, user will not be the group
    administrator.
    :var status: If true, user will be active GroupMember; otherwise, user will not be active GroupMember.
    :var isDefaultRole: If true, the given role will be the default role for the user; otherwise the given role will
    not be the default role for the user.
    """
    isGroupAdmin: bool = False
    status: bool = False
    isDefaultRole: bool = False


@dataclass
class UserManagementResponse(TcBaseObj):
    """
    This structure holds a structure of User and associated Role and Group objects along with ServiceData.
    
    :var userRoleGrpStructs: A list of structures of User and associated Role and Group objects.
    :var serviceData: The Service Data with the partial error information.
    """
    userRoleGrpStructs: List[UserRoleGroupStructure] = ()
    serviceData: ServiceData = None


@dataclass
class UserRoleGroupStructure(TcBaseObj):
    """
    A structure of User and associated Role and Group objects.
    
    :var users: A list of User objects to be removed from given Group and Role.
    :var grp: A Group object from which User is to be removed.
    :var role: A Role object from which User is to be removed.
    """
    users: List[User] = ()
    grp: Group = None
    role: Role = None


@dataclass
class AddUsersAsGroupMembersStructure(TcBaseObj):
    """
    A structure of User object and the associated Role and Group objects.
    
    :var clientId: Identifier that helps the client to track the object(s) created and modified, required; should be
    unique for the input set.
    :var usersToAdd: A list of existing users to be added to the Group with specified Role.
    :var usersToCreateAndAdd:  A list of users to be created and added to the Group with specified Role.
    :var grp: Group for the group members.
    :var role: The Role for the group members.
    """
    clientId: str = ''
    usersToAdd: List[ExistingUserDetailsForGroupMember] = ()
    usersToCreateAndAdd: List[NewUserDetailsForGroupMember] = ()
    grp: Group = None
    role: Role = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Core._2007_06.ProjectLevelSecurity import GetFilteredProjectDataInputData
from tcsoa.gen.Internal.Core._2013_05.ProjectLevelSecurity import GetFilteredProjectObjectsResponse
from tcsoa.gen.Internal.Core._2013_05.Licensing import GetLicenseBundlesResponse
from tcsoa.gen.Internal.Core._2013_05.PresentationManagement import GetStylesheetPref, GetSharedCommonClientFilesResponse, StylesheetInputData, GetStylesheetResponse
from typing import List
from tcsoa.gen.Internal.Core._2009_10.Thumbnail import SearchOrders
from tcsoa.base import TcService
from tcsoa.gen.Internal.Core._2013_05.Thumbnail import ThumbnailFileTicketsResponse2


class PresentationManagementService(TcService):

    @classmethod
    def getSharedCommonClientFiles(cls, prefNames: List[str]) -> GetSharedCommonClientFilesResponse:
        """
        This service returns the dataset names, last modified dates, and file tickets to download for Zip datasets as
        defined by the input list of preference names, or if empty input, the HTMLStylesheetSharedLibs preference is
        used.   The preference indicates the list of Zip datasets by dataset name, and there must exist only one
        dataset of that type and name in the system which is owned by a "dba" user.  If this criteria isn't met for a
        dataset defined in the preference, then a partial error is added to the service data.
        
        Use cases:
        1.    The user chooses the summary tab in RichClient.
        2.    Richclient detects that the shared common client files which are required in order to render HTML are not
        downloaded.
        3.    RichClient invokes the getSharedCommonClientFiles operation and downloads the list of file tickets to a
        folder, and caches the dataset names and last modified dates.
        4.    RichClient invokes the getStylesheet operation, receives HTML stylesheet and renders it to the User.
        5.    On a subsequent close/reopen of the Summary View, RichClient reinvokes the getSharedCommonClientFiles
        operation, and receives the list of file tickets, dataset names, and last modified dates.   The RichClient
        downloads any new datasets and any datasets which have a newer timestamp than what was cached.
        6.    Rich client invokes the getStylesheet operation, reccieves new HTML stylesheet output, and renders it to
        the User.
        """
        return cls.execute_soa_method(
            method_name='getSharedCommonClientFiles',
            library='Internal-Core',
            service_date='2013_05',
            service_name='PresentationManagement',
            params={'prefNames': prefNames},
            response_cls=GetSharedCommonClientFilesResponse,
        )

    @classmethod
    def getStylesheet(cls, pref: GetStylesheetPref, inputData: List[StylesheetInputData]) -> GetStylesheetResponse:
        """
        This service converts XRT stylesheets to HTML stylesheets and returns the stylesheet data to the client.   If
        the Teamcenter preference "UsePropertyStylesheetPlatformRenderer" is set to "true", then only XRT is returned
        and no attempt to convert/return HTML is made.    This service accepts a "returnHTMLOnly" preference which
        indicates that the caller can only consume HTML, and overrides the "UsePropertyStylesheetsPlatformRenderer"
        preference, to enable conversion to HTML.  This operation only returns XRT or HTML, never both values to the
        client.
        
        If the XRT to be presented in UI contains objectSet elements, which display an object's children in UI, then
        those children are returned in the output childrenMap container, and those child thumbnail file tickets are
        added to the FileTicketMap, as required.
        
        Common uses for this operation would be to retrieve stylesheet data in either XRT or HTML format for a selected
        object in the UI, in order to present that object to the user.   
        
        Use cases:
        Use case 1:
        1.    The user chooses the summary tab in RichClient.
        2.    The user selects an object in the TCComponent navigator
        3.    The getStylesheet operation is invoked and the resulting stylesheet is presented in the UI.
        
        Use case 2:
        1.    The user selects an object in the TCComponent navigator in the UI.
        2.    The user chooses to view properties for that object.
        3.    The getStylesheet operation is invoked and the resulting stylesheet is presented in the UI.
        """
        return cls.execute_soa_method(
            method_name='getStylesheet',
            library='Internal-Core',
            service_date='2013_05',
            service_name='PresentationManagement',
            params={'pref': pref, 'inputData': inputData},
            response_cls=GetStylesheetResponse,
        )


class ThumbnailService(TcService):

    @classmethod
    def getThumbnailFileTickets2(cls, businessObjects: List[BusinessObject], searchOrders: SearchOrders) -> ThumbnailFileTicketsResponse2:
        """
        Given a list of business objects and relation/dataset type search orders, gets the valid thumbnail file tickets
        for the list of business objects. These file tickets can be used later to download the thumbnail images using
        File Client Cache (FCC). For each business object, it will get a list of valid thumbnail file tickets.
        """
        return cls.execute_soa_method(
            method_name='getThumbnailFileTickets2',
            library='Internal-Core',
            service_date='2013_05',
            service_name='Thumbnail',
            params={'businessObjects': businessObjects, 'searchOrders': searchOrders},
            response_cls=ThumbnailFileTicketsResponse2,
        )


class ProjectLevelSecurityService(TcService):

    @classmethod
    def getFilteredObjectsInProject(cls, input: List[GetFilteredProjectDataInputData]) -> GetFilteredProjectObjectsResponse:
        """
        This operation returns the UIDs that match the requested filter criteria for the requested projects. If no
        objects are found matching a given filter criteria or if project ID is invalid, then project's ID along with
        empty list will be returned.
        """
        return cls.execute_soa_method(
            method_name='getFilteredObjectsInProject',
            library='Internal-Core',
            service_date='2013_05',
            service_name='ProjectLevelSecurity',
            params={'input': input},
            response_cls=GetFilteredProjectObjectsResponse,
        )


class LicensingService(TcService):

    @classmethod
    def getLicenseBundles(cls, licenseServerNames: List[str]) -> GetLicenseBundlesResponse:
        """
        This operation retrieves information about the license bundles from the license file associated with each
        license server specified by the input parameter licenseServerNames. The information for a license bundle
        includes the bundle name, the associated base license feature key, the count of purchased licenses and its
        expiry date. If a license server does not exist with the specified name or if the license server is not online,
        a partial error is returned for that license server.
        """
        return cls.execute_soa_method(
            method_name='getLicenseBundles',
            library='Internal-Core',
            service_date='2013_05',
            service_name='Licensing',
            params={'licenseServerNames': licenseServerNames},
            response_cls=GetLicenseBundlesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, RevisionRule
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ColumnConfig(TcBaseObj):
    """
    Output Column configuration computed by server
    
    :var columnConfigId: Column configuration ID.
    :var operationType: Operation type used for determining columns. Supported values are: "Union" and "Intersection".
    :var columns: List of available columns
    :var typesForArrange: The valid Teamcenter types used for fetching the columns.
    """
    columnConfigId: str = ''
    operationType: str = ''
    columns: List[ColumnDefInfo] = ()
    typesForArrange: List[str] = ()


@dataclass
class ColumnConfigInput(TcBaseObj):
    """
    Input required to compute column configuration information
    
    :var clientName: The name of a client application, as represented by an instance of Fnd0Client in the Teamcenter
    database. This value must match the value of fnd0ClientName property. For example: The client name for Active
    Workspace is "AWClient".
    :var hostingClientName: Specifies the name of a hosting client application, as represented by an instance of
    Fnd0Client, in the Teamcenter databases. This value must match a value of the fnd0ClientName property. For example:
    If client A is integrated with client B and the user can invoke client B commands from within client A, the input
    would specify client A as hosting client and client B as the client. If the caller wanted native commands for
    client A, client A would be specified as client and hosting client would be empty.
    :var clientScopeURI: The unique name of the client scope containing column configurations.
    :var operationType: The operation that needs to be applied to finalize the columns to be returned. Valid values are:
    "Intersection" - Gets the intersection of the columns for the types found in search results.         
    "Union" - Gets all the columns for the types found in search results.
    :var columnsToExclude: List of columns which should be excluded from the final list being returned. The
    value provided should be in the format "TypeName.PropertyName". Both type name and property name should be internal
    values. For example: ItemRevision.sequence_id, where '.' is the delimiter.
    """
    clientName: str = ''
    hostingClientName: str = ''
    clientScopeURI: str = ''
    operationType: str = ''
    columnsToExclude: List[str] = ()


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Detailed information about individual columns
    
    :var displayName: The display name for the property to be displayed in the column header.
    :var assosiatedTypeName: The business object type for the value displayed in the column. This can be any valid
    Teamcenter business object type.
    :var propertyName: The internal property name for the value displayed in the column
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between 1 and 500
    :var columnOrder: The column order value is used to arrange the columns in order
    :var hiddenFlag: If true, the column to be hidden on the client user interface
    :var sortPriority: Sort priority set on column helps identify the order in which the columns should be used during
    sort. Sort priority value will be zero for columns not marked for sorting
    :var sortDirection: How the columns are sorted. Supported values are: "Ascending" and "Descending". This value will
    be empty if the column is not marked for sorting.
    """
    displayName: str = ''
    assosiatedTypeName: str = ''
    propertyName: str = ''
    pixelWidth: int = 0
    columnOrder: int = 0
    hiddenFlag: bool = False
    sortPriority: int = 0
    sortDirection: str = ''


@dataclass
class Cursor(TcBaseObj):
    """
    This gives information about returned page.
    
    :var resultsReturned: Number of results returned so far
    :var endReached: If true, end is reached; otherwise, end is not yet reached and there are more results to be
    returned.
    :var startUIDLevel: The startUIDLevel helps where used to determine the previous page to return
    :var endUIDLevel: The endUIDLevel helps where used to determine the next page to return
    :var startObjectUid: The startObject helps where used to determine the previous page to return. null allowed.
    :var endObjectUid: The endObject helps where used to determine the next page to return. null allowed
    """
    resultsReturned: int = 0
    endReached: bool = False
    startUIDLevel: int = 0
    endUIDLevel: int = 0
    startObjectUid: str = ''
    endObjectUid: str = ''


@dataclass
class ResultNode(TcBaseObj):
    """
    Contains parent UID and boolean indicating if parent is further parents.
    
    :var resultObject: BusinessObject under which input ItemRevision is created as child component.
    :var hasParent: If true, there are parents for this node; otherwise, this node does not have parents and is a root
    node.
    """
    resultObject: BusinessObject = None
    hasParent: bool = False


@dataclass
class WhereUsedInput(TcBaseObj):
    """
    Input required for getWhereUsedInfo SOA operation
    
    :var inputObject: The ItemRevision object for which &lsquo;where used&rsquo; information is to be retrieved.  This
    is mandatory input.
    :var revisionRule: The RevisionRule object to configure parents. If null, default RevisionRule object is used from
    preference: "TC_config_rule_name".
    :var additionalInfo: Additional set of information required by operation in key value pair. Future scope.  This
    would be a map of string to vector of string.
    :var cursorInfo: A map (BusinessObject, Cursor) of child BusinessObject to its cursor information.
    :var pageSize: Number of parent BusinessObjects to be returned.
    :var columnConfigInput: Column configuration input
    """
    inputObject: BusinessObject = None
    revisionRule: RevisionRule = None
    additionalInfo: AdditionalInfo = None
    cursorInfo: CursorMap = None
    pageSize: int = 0
    columnConfigInput: ColumnConfigInput = None


@dataclass
class WhereUsedOutput(TcBaseObj):
    """
    Output from getWhereUsedInfo SOA operation
    
    :var childToParentMap: A map (BusinessObject, list of ResultNode) of child BusinessObject as key and list of its
    parent ResultNodes as value.
    :var cursorInfo: A map (BusinessObject, Cursor) of Child BusinessObject to its Cursor information like, number of
    results returned for that child and if end has reached for that child.
    :var additionalInfo: Additional set of information SOA operation wants to send to client
    :var columnConfigOutput: Column configuration containing  BusinessObject type, property internal name, display
    name, column width based input column configuration.
    :var serviceData: ServiceData containing added, updated, deleted objects along with partial errors, if any.
    """
    childToParentMap: ChildToParentMap = None
    cursorInfo: CursorMap = None
    additionalInfo: AdditionalInfo = None
    columnConfigOutput: ColumnConfig = None
    serviceData: ServiceData = None


"""
Additional set of information SOA operation wants to send to client
"""
AdditionalInfo = Dict[str, List[str]]


"""
A map (BusinessObject, list of ResultNode) of child BusinessObject as key and list of its parent ResultNodes as value.
"""
ChildToParentMap = Dict[BusinessObject, List[ResultNode]]


"""
A map (BusinessObject, Cursor) of Child BusinessObject to its Cursor information like, number of results returned for that child and if end has reached for that child.
"""
CursorMap = Dict[str, Cursor]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def unloadObjects(cls, objsToUnload: List[BusinessObject]) -> ServiceData:
        """
        This operation unloads Business Objects. If input contains one or more Business Objects then it will call
        AOM_unload for each object otherwise it will call unloadAll  to unload all the objects. 
        
        Note that unloadAll will unload both C++ along with POM objects.
        """
        return cls.execute_soa_method(
            method_name='unloadObjects',
            library='Core',
            service_date='2008_05',
            service_name='DataManagement',
            params={'objsToUnload': objsToUnload},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Requirementsmanagement._2008_06.RequirementsManagement import GetContentInput1
from tcsoa.gen.Requirementsmanagement._2007_01.RequirementsManagement import GetRichContentResponse
from typing import List
from tcsoa.base import TcService


class RequirementsManagementService(TcService):

    @classmethod
    def getRichContent(cls, inputs: List[GetContentInput1]) -> GetRichContentResponse:
        """
        The SOA operation is responsible for retrieving contents of objects of type SpecElement. It can be used to view
        content of SpecElement in word. SOA also provides capability to view contents by applying default templates.
        
        Use cases:
        User can open requirements in word using default templates or they can open content (body text) in word for
        view and edit purpose.
        
        Exceptions:
        >If there is any error during generating transient file read ticket due to a configuration issue at the server,
        then the operation throws a service exception. Example- If the transient volume directory is not configured at
        the server then the FMS fails to generate a file at the server and subsequent file download operation fails. In
        such situation a service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='getRichContent',
            library='Requirementsmanagement',
            service_date='2008_06',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=GetRichContentResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Fnd0Client, Fnd0AbstractCommand, Fnd0UIConfigCollectionRel, Fnd0ClientScope
from tcsoa.gen.AWS2._2016_12.UiConfig import ClientInput, ScopeInput, CommandCollectionInfo
from tcsoa.gen.Internal.AWS2._2016_03.UiConfig import CommandContextInfo
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetUIConfigInput(TcBaseObj):
    """
    Contains input information required to retrieve UI configurations from the Teamcenter database.
    
    :var clientScopeURIs: A list of client scope URIs representing, for example location.sublocation in Active
    Workspace. (Fnd0ClientScope::fnd0ClientScopeURI). If empty, the UI Configuration for all client scopes are returned.
    :var scope: The scope of the desired UI configuration information. This includes the name of the scope (i.e. a
    user, group or role) and scope query parameter information.
    :var client: Client information including client name and hosting client name.
    :var commandContextInfo: A list of command context name value pairs, Supprted key/value pairs are :
    isHosted"="true|false", - if true , active workspace clinet is hosted in another application such as NX. If false,
    active workspace client is stand alone. "productContext"=productContextUid.
    """
    clientScopeURIs: List[str] = ()
    scope: ScopeInput = None
    client: ClientInput = None
    commandContextInfo: List[CommandContextInfo] = ()


@dataclass
class GetUIConfigResponse(TcBaseObj):
    """
    This structure returns information to the client about column configuration and command applicability. The
    ServiceData contains information about errors encountered during processing.
    
    :var serviceData: ServiceData structure containing errors and command, command collection and icon objects. If
    there is an error retrieving the configuration information, the error added to the ServiceData as a partial error.
    :var uiConfigInfo: List of configuration information including command and column information.
    """
    serviceData: ServiceData = None
    uiConfigInfo: List[ClientConfigurations] = ()


@dataclass
class ClientConfigurations(TcBaseObj):
    """
    This structure contains command and column configuration data for the indicated client.
    
    :var client: The client for which the configurations are applicable. It is to match the clientName in ClientInput
    SOA struture.
    :var columnConfigurations: A list of column configurations.
    :var commandConfigurations: A list of command configurations.
    """
    client: str = ''
    columnConfigurations: List[ColumnConfigData] = ()
    commandConfigurations: List[CommandConfigData] = ()


@dataclass
class ColumnConfig(TcBaseObj):
    """
    This structure contains information for a column configuration within a client scope URI. It contains a unique
    column config id, a list of column definition information, and the default sort direction.
    
    :var columnConfigId: The unique identifier of the column.
    :var sortDirection: How the columns are sorted. Valid values are Ascending and Descending.
    :var operationType: The operation that was used to finalize the columns to be returned back.
    Valid values are:  "Intersection", "Union" and "Configured".
    :var columns: Ordered list of column details.
    """
    columnConfigId: str = ''
    sortDirection: str = ''
    operationType: str = ''
    columns: List[ColumnDefInfo] = ()


@dataclass
class ColumnConfigData(TcBaseObj):
    """
    This structure returns information about the column configuration definitions for a scope, hosting client and
    client scope.
    
    :var scope: The scope for which the column data is applicable.
    :var scopeName: The scope name for which the column data is applicable.
    :var hostingClient: The name of hosting client for which the list of column configurations is applicable. This
    value must correspond to the value of the property fnd0ClientName for an Fnd0Client object in the Teamcenter
    database.
    :var clientScopeURI: The client scope for which the list of column configurations is applicable. This must match a
    value of fnd0ClientScopeURI which is the unique identifier of a client scope (Fnd0ClientScope).
    :var columnConfigurations: List of column configuration details.
    """
    scope: str = ''
    scopeName: str = ''
    hostingClient: str = ''
    clientScopeURI: str = ''
    columnConfigurations: List[ColumnConfig] = ()


@dataclass
class ViewModelPropertyDescriptor(TcBaseObj):
    """
    Represents the property descriptor details for a ViewModelProperty.
    
    :var srcObjectTypeName: Type name of the source object on which the property is defined.
    :var propertyName: Name of the property.
    :var valueType: Indicates the value type for the property. Valid value types are: 
    1 - Char
    2 - Date
    3 - Double
    4 - Float
    5 - Integer
    6 - Boolean
    7 - Short Integer
    8 - String
    9 - Type Reference
    10 - UnTyped Reference
    11 - External Reference
    12 - Note
    13 - Typed Relation
    14 - UnTyped Relation
    :var isArray: Has value as true in case of array properties and value as false for single valued properties.
    :var propConstants: A map(string,string) consisting of propertyConstant name as key and  and value being the
    property constant value.
    :var displayName: Display name of the property.
    :var lovCategory: Indicates the LOV category, if the property is attached to LOV.
    :var lov: LOV object reference associated with the property.
    :var maxArraySize: Maximimum number of elements allowed in case of VLA properties.
    :var maxLength: Max allowed length for the property
    :var propertyType: Indicates the property type. Valid property types are: 
    1 - attribute. 
    2 - Reference Property. 
    3 - Relation Property. 
    4 - Compound Property. 
    5 - Runtime Property.
    6 - Operation input.
    """
    srcObjectTypeName: str = ''
    propertyName: str = ''
    valueType: int = 0
    isArray: bool = False
    propConstants: PropertyConstantsMap = None
    displayName: str = ''
    lovCategory: int = 0
    lov: BusinessObject = None
    maxArraySize: int = 0
    maxLength: int = 0
    propertyType: int = 0


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Contains details about a specific column. This includes the type of object for which the column is applicable, the
    name of the property displayed in the column, a flag indicating if the column should be used to order information
    displayed in the client and pixel width and the display name of the property.
    
    :var sortByFlag: If true, the column is used to sort the information displayed to the user; otherwise, not used.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are values between 1 and 500.
    :var columnOrder: The column order value is used to arrange the columns in order.
    :var hiddenFlag: True if the column should be hidden on the client user interface.
    :var sortPriority: Sort priority set on column helps identify the order in which the columns should be used during
    sort. Sort priority value will be zero for columns not marked for sorting.
    :var colDefSortDirection: How the columns are sorted. Valid values are: "Ascending" and "Descending". This value
    will be empty if the column is not marked for sorting.
    :var propDescriptor: List of ViewModelPropertyDescriptors required to render the object properties in AW client.
    :var columnSrcType: Valid Teamcenter type for which the column property has been defined
    """
    sortByFlag: bool = False
    pixelWidth: int = 0
    columnOrder: int = 0
    hiddenFlag: bool = False
    sortPriority: int = 0
    colDefSortDirection: str = ''
    propDescriptor: ViewModelPropertyDescriptor = None
    columnSrcType: str = ''


@dataclass
class CommandConfigData(TcBaseObj):
    """
    This structure returns information about the command configuration definitions for a client scope URI.
    
    :var scopeName: This structure returns information about the command configuration definitions for a client scope
    URI.
    :var hostingClient: The hosting Fnd0Client for which the command data is applicable.
    :var clientScope: The Fnd0ClientScope containing the list of command configurations.
    :var cmdCollections: A list of all top level and child command collections in the client scope.
    :var cmdCollectionIndex: A list of index values into cmdCollections list for top level command collections in the
    client scope.
    :var commands: A list of all Fnd0AbstractCommand objects accessible in the client scope.
    :var commandCollectionRels: A list of all Fnd0UIConfigCollectionRel objects that associate top level command
    collections with client scope.
    :var visibleCommands: A list of visible Fnd0AbstractCommand objects accessible in the client scope.
    """
    scopeName: str = ''
    hostingClient: Fnd0Client = None
    clientScope: Fnd0ClientScope = None
    cmdCollections: List[CommandCollectionInfo] = ()
    cmdCollectionIndex: List[int] = ()
    commands: List[Fnd0AbstractCommand] = ()
    commandCollectionRels: List[Fnd0UIConfigCollectionRel] = ()
    visibleCommands: List[Fnd0AbstractCommand] = ()


"""
Map to store the property constant name and value pair.
"""
PropertyConstantsMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import AppInterface
from tcsoa.gen.Ai._2006_03.Ai import ApplicationRef
from tcsoa.gen.Ai._2008_06.Ai import Configuration
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetMappedAppRefsInput(TcBaseObj):
    """
    GetMappedAppRefsInput structure contains a Application Reference representing the object for which Application
    References of different Application name are needed, the Configuration to be used if Application Reference is of
    Teamcenter PSOccurrenceThread chain, and a list of Application names.
    
    :var configuration: A optional element only used if the appRef is of "Teamcenter" application and is a
    PSOccurrenceThread chain. If not used, the useDefaultRevistionRule member should be set to false and existingWindow
    should be set to NULL.
    :var appRefs: A list of 3-tuple each consisting of application name, label string and version string to represent
    the objects for which Application References are needed. For each Application Reference in the list, there will be
    appNames number of Application References.
    :var appNames: A list of strings representing the names of Applications for which the "appRef" values are needed.
    Atleast one value must be provided. Valid value for Teamcenter is "Teamcenter".
    """
    configuration: Configuration = None
    appRefs: List[ApplicationRef] = ()
    appNames: List[str] = ()


@dataclass
class GetMappedAppRefsResponse(TcBaseObj):
    """
    GetMappedAppRefsResponseElement returns the Application References for each input appref for each input appname.
    The vector is laid out sequencially - all the apprefs for each appName following each other.
    
    :var appRefs: A list of GetMappedRefsResponseElement structures.
    :var serviceData: Service data capturing partial errors using the input array index as client id.
    """
    appRefs: List[GetMappedAppRefsResponseElement] = ()
    serviceData: ServiceData = None


@dataclass
class GetMappedAppRefsResponseElement(TcBaseObj):
    """
    Element corresponding to each element in the input containing the details of the desired Application Reference. The
    layout of the vector corresponds to appRef values for each appName in sequence.
    
    :var appRefs: A list of Application Reference elements corresponding to the Application names specified in the
    input element.
    """
    appRefs: List[ApplicationRef] = ()


@dataclass
class CreateAppInterfaceRecordInput(TcBaseObj):
    """
    CreateAppInterfaceRecordInput structure contains a AppInterface object and PLMXML labels for which corresponding
    RecordObjects are to be created and associated with the MasterRecord of the input AppInterface object.
    
    :var ai: The ApplicationInterface object whose MasterRecord is to be modified with the newly created RecordObjects
    based on the labels.
    :var labels: A list of strings in the label format of PLMXML Application Reference element.
    """
    ai: AppInterface = None
    labels: List[str] = ()


@dataclass
class CreateAppInterfaceRecordOutput(TcBaseObj):
    """
    CreateAppInterfaceRecordOutput returns the PLMXML style labels for which the RecordObjects could not be created and
    the corresponding reason for failure.
    
    :var failedLabels: A list of labels for which  RecordObjects could not be created.
    :var reasons: A list of strings each indicating the reason for failure.
    """
    failedLabels: List[str] = ()
    reasons: List[str] = ()


@dataclass
class CreateAppInterfaceRecordsResponse(TcBaseObj):
    """
    CreateAppInterfaceRecordsResponse structure contains a vector of CreateAppInterfaceRecordOutput, the size of which
    matches the input vector. It also includes the standard serviceData object.
    
    :var output: A list of CreateAppInterfaceRecordOutput structures.
    :var serviceData: Service data capturing partial errors using the input array index as client id.
    """
    output: List[CreateAppInterfaceRecordOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.Configurator._2016_09.ConfiguratorManagement import AvailabilityOutput, AvailabilityInput
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def getOptionValueAvailability(cls, input: AvailabilityInput) -> AvailabilityOutput:
        """
        This operation computes the available features within a set of requested families and models. The parameters
        criteriaExpression and familiesToTest are used to pass in the input expression representing a model and
        families of interest for which available features are requested. The operation response returns the available
        product variability for all the features in the requested families, if the features satisfy the input
        expression in criteriaExpression and the constraint(s) present in the system. The input parameter
        criteriaExpression is required to contain expression representing models. Iif they don't the the
        criteriaExpression is empty the operation throws an  error, . The input parameter familiesToTest is required to
        contain list of Family. If the requested family list is empty, the operation throws an error.
        
        Use cases:
        Consider the following Variability data in a Configurator Ccontext itemsay "MyProductItem":
        
        Model Family    Features
        M
        - m1
        - m2
        - m3
        
        
        
        Family    Features   
        A
        - a1
        - a2
        - a3
        
        
        
        B
        - b1
        - b2
        - b3
        
        
        
        C
        - c1
        - c2
        - c3
        
        
        
        Rules:
        - a1 is available with m1:  This is anavailability rule stating that the configuration feature a1 (of the
        family A) is available with the model m1 (of model family M).
        - a2 is available with m1:  This is an availability rule stating that the configuration feature a2 (of the
        family A) is available with the model m1 (of model family M).
        - b1 is available with m2:  This is an availability rule stating that the configuration feature b1 (of the
        family B) is available with the model m2 (of model family M).
        
        
        
        
        Use case 1:
        Operation getOptionValueAvailability is invoked with AvailabilityInput populated as:
        Configurator Context item = "MyProductItem"
        criteriaExpression = { M=m1}
        familiesToTest = {A}
        applyConstraints = 1(enable configurator constraint evaluation)
        
        Response -  
        For Model M1 - a1(Available ) , a2(Available ), a3( Not Available )
        
        Use case 2:
        Operation getOptionValueAvailability is invoked with AvailabilityInput populated as:
        Configurator Context item = "MyProductItem"
        CriteriaExpression = { M=m1, M=m2 }
        FamiliesToTest = { A,B }
        applyConstraints = 1
        
        Response - 
        For Model M1 - a1(Available ) , a2(Available ), a3( Not Available ), b1(Not Available) , b2(Not Available), b3(
        Not Available )
        For Model M2 - a1(Not Available) , a2(Not Available), a3( Not Available ), b1(Available) , b2(Not Available),
        b3( Not Available )
        
        Use case 3:
        Operation getOptionValueAvailability is invoked with AvailabilityInput populated as:
        Configurator Context item = "MyProductItem"
         CriteriaExpression = { M=m1, M=m2 }
        FamiliesToTest =  {}
        applyConstraints = 1
        
        Response - 
        The parameter serviceData will contain appropriate error(s), because familiesToTest is empty
        
        Use case 4:
        Operation getOptionValueAvailability is invoked with AvailabilityInput populated as:
        Configurator Context item = "MyProductItem"
         CriteriaExpression = { }
        FamiliesToTest = { A,B }
        applyConstraints = 1
        
        Response - 
        The parameter serviceData will contain  appropriate error(s), because CriteriaExpression is empty.
        """
        return cls.execute_soa_method(
            method_name='getOptionValueAvailability',
            library='Internal-Configurator',
            service_date='2016_09',
            service_name='ConfiguratorManagement',
            params={'input': input},
            response_cls=AvailabilityOutput,
        )



from tcsoa.gen.Internal_StructureManagement._2007_06.services import GlobalAlternateService as imp0
from tcsoa.gen.Internal_StructureManagement._2016_03.services import StructureVerificationService as imp1
from tcsoa.gen.Internal_StructureManagement._2010_04.services import BOMMarkupService as imp2
from tcsoa.gen.Internal_StructureManagement._2019_06.services import VariantManagementService as imp3
from tcsoa.gen.Internal_StructureManagement._2015_10.services import VariantManagementService as imp4
from tcsoa.gen.Internal_StructureManagement._2008_06.services import StructureService as imp5
from tcsoa.gen.Internal_StructureManagement._2007_06.services import PublishByLinkService as imp6
from tcsoa.gen.Internal_StructureManagement._2011_06.services import StructureService as imp7
from tcsoa.gen.Internal_StructureManagement._2009_10.services import EffectivitiesManagementService as imp8
from tcsoa.gen.Internal_StructureManagement._2014_12.services import StructureVerificationService as imp9
from tcsoa.gen.Internal_StructureManagement._2020_05.services import RevisionRuleAdministrationService as imp10
from tcsoa.gen.Internal_StructureManagement._2016_10.services import EffectivityService as imp11
from tcsoa.gen.Internal_StructureManagement._2013_05.services import StructureExpansionLiteService as imp12
from tcsoa.gen.Internal_StructureManagement._2013_12.services import StructureExpansionLiteService as imp13
from tcsoa.gen.Internal_StructureManagement._2017_05.services import StructureExpansionLiteService as imp14
from tcsoa.gen.Internal_StructureManagement._2012_02.services import StructureVerificationService as imp15
from tcsoa.gen.Internal_StructureManagement._2011_06.services import IncrementalChangeService as imp16
from tcsoa.gen.Internal_StructureManagement._2017_05.services import StructureVerificationService as imp17
from tcsoa.gen.Internal_StructureManagement._2011_06.services import VariantManagementService as imp18
from tcsoa.gen.Internal_StructureManagement._2014_12.services import BrokenLinksService as imp19
from tcsoa.gen.Internal_StructureManagement._2007_12.services import BrokenLinksService as imp20
from tcsoa.gen.Internal_StructureManagement._2018_11.services import StructureVerificationService as imp21
from tcsoa.gen.Internal_StructureManagement._2012_09.services import StructureVerificationService as imp22
from tcsoa.gen.Internal_StructureManagement._2018_11.services import MassUpdateService as imp23
from tcsoa.gen.Internal_StructureManagement._2007_06.services import RestructureService as imp24
from tcsoa.gen.Internal_StructureManagement._2017_05.services import StructureLiteConversionService as imp25
from tcsoa.gen.Internal_StructureManagement._2008_05.services import RestructureService as imp26
from tcsoa.gen.Internal_StructureManagement._2014_12.services import RestructureService as imp27
from tcsoa.gen.Internal_StructureManagement._2008_03.services import StructureService as imp28
from tcsoa.gen.Internal_StructureManagement._2010_09.services import StructureService as imp29
from tcsoa.gen.Internal_StructureManagement._2007_06.services import RedliningService as imp30
from tcsoa.gen.Internal_StructureManagement._2008_05.services import StructureService as imp31
from tcsoa.base import TcService


class GlobalAlternateService(TcService):
    addRelatedGlobalAlternates = imp0.addRelatedGlobalAlternates
    listGlobalAlternates = imp0.listGlobalAlternates
    removeRelatedGlobalAlternates = imp0.removeRelatedGlobalAlternates
    setPreferredGlobalAlternate = imp0.setPreferredGlobalAlternate


class StructureVerificationService(TcService):
    alignMatchedCandidates = imp1.alignMatchedCandidates
    createOrUpdatePropagationDetails = imp9.createOrUpdatePropagationDetails
    createOrUpdateReviewStatus = imp9.createOrUpdateReviewStatus
    findMatchingCandidates = imp1.findMatchingCandidates
    findReviewStatus = imp9.findReviewStatus
    getActivitiesComparisonDetails = imp15.getActivitiesComparisonDetails
    getAttachmentComparisonDetails = imp17.getAttachmentComparisonDetails
    getMountAttachComparisonDetails = imp21.getMountAttachComparisonDetails
    getPropertyPropagationStatusDetails = imp9.getPropertyPropagationStatusDetails
    getStructureChangeDetails = imp9.getStructureChangeDetails
    getStructureChangeImpactedLines = imp9.getStructureChangeImpactedLines
    getToolRequirementComparisonDetails = imp15.getToolRequirementComparisonDetails
    getValidCriteria = imp22.getValidCriteria


class BOMMarkupService(TcService):
    applyBOMMarkup = imp2.applyBOMMarkup
    createBOMMarkup = imp2.createBOMMarkup
    savePendingEditsAsMarkup = imp2.savePendingEditsAsMarkup


class VariantManagementService(TcService):
    applyRollupVariantConfiguration = imp3.applyRollupVariantConfiguration
    applyVariantConfiguration = imp4.applyVariantConfiguration
    getBOMVariantConfigOptions = imp18.getBOMVariantConfigOptions
    getModularOptionsForBom = imp18.getModularOptionsForBom


class StructureService(TcService):
    copyRecursively = imp5.copyRecursively
    createOrSavePSBOMViewRevision = imp7.createOrSavePSBOMViewRevision
    findHighestFindNumberInExpand = imp5.findHighestFindNumberInExpand
    getAvailableViewTypes = imp7.getAvailableViewTypes
    resequence = imp28.resequence
    resequence2 = imp29.resequence
    syncAlignedOccurrences = imp31.syncAlignedOccurrences


class PublishByLinkService(TcService):
    createIDCWindowForDesignAsm = imp6.createIDCWindowForDesignAsm


class EffectivitiesManagementService(TcService):
    createOrUpdateEffectivites = imp8.createOrUpdateEffectivites
    getEffectivityGrpRevList = imp8.getEffectivityGrpRevList
    setEndItemEffectivityGroups = imp8.setEndItemEffectivityGroups


class RevisionRuleAdministrationService(TcService):
    createOrUpdateRevisionRule = imp10.createOrUpdateRevisionRule
    getRevisionRuleInfo = imp10.getRevisionRuleInfo


class EffectivityService(TcService):
    cutbackUnitOccurrenceEffectivity = imp11.cutbackUnitOccurrenceEffectivity
    getUnitNetOccurrenceEffectivity = imp11.getUnitNetOccurrenceEffectivity


class StructureExpansionLiteService(TcService):
    expandBasedOnOccurrenceList = imp12.expandBasedOnOccurrenceList
    expandBasedOnOccurrenceList2 = imp13.expandBasedOnOccurrenceList2
    expandNext = imp12.expandNext
    expandNext2 = imp13.expandNext2
    expandNext3 = imp14.expandNext3
    getUndelivered = imp12.getUndelivered
    getUndelivered2 = imp13.getUndelivered2
    getUndelivered3 = imp14.getUndelivered3
    unloadBelow = imp12.unloadBelow
    unloadBelow2 = imp13.unloadBelow2


class IncrementalChangeService(TcService):
    getAttachmentChanges = imp16.getAttachmentChanges
    getAttributeChanges = imp16.getAttributeChanges
    getParentAndChildComponents = imp16.getParentAndChildComponents
    getPredecessorChanges = imp16.getPredecessorChanges
    getStructureChanges = imp16.getStructureChanges


class BrokenLinksService(TcService):
    getBrokenLinkAndReplacements = imp19.getBrokenLinkAndReplacements
    getBrokenLinkInfoWithFixOpt = imp20.getBrokenLinkInfoWithFixOpt
    repairBrokenLinks = imp20.repairBrokenLinks


class MassUpdateService(TcService):
    hasActiveMarkupAssociated = imp23.hasActiveMarkupAssociated
    saveImpactedAssemblies = imp23.saveImpactedAssemblies


class RestructureService(TcService):
    insertLevel = imp24.insertLevel
    moveNode = imp24.moveNode
    removeLevel = imp24.removeLevel
    replaceInContext = imp26.replaceInContext
    replaceItems = imp27.replaceItems
    splitOccurrence = imp24.splitOccurrence


class StructureLiteConversionService(TcService):
    liteBOMLinesToBOMLines = imp25.liteBOMLinesToBOMLines


class RedliningService(TcService):
    revertAllPendingEdits = imp30.revertAllPendingEdits
    revertPendingEdits = imp30.revertPendingEdits



from __future__ import annotations

from tcsoa.gen.Internal.Workflow._2017_11.Workflow import CreateWorkflowInput
from tcsoa.base import TcService


class WorkflowService(TcService):

    @classmethod
    def createWorkflowAsync(cls, inputData: CreateWorkflowInput) -> None:
        """
        Operation to create a workflow process in an asynchrous fashion. A workflow process is the running instance of
        a workflow template. This operation can be used to create a workflow process or sub process at a local or a
        remote site from a given workflow template in an asynchrous fashion.
        
        Use cases:
        Creating a workflow process in asynchronous mode improves the performance of the workflow creation by
        submitting the create request for background processing. This is especially useful when workflow is created
        from a very large workflow template. Creating a workflow using a large workflow template can take long time,
        since it involves creation of many objects, such as: tasks (EPMTask), signoff objects (Signoff),  and process
        instance (EPMJob). So, for cases that use large workflow templates, this SOA can be used to submit the workflow
        creation in background to improve performance.
        """
        return cls.execute_soa_method(
            method_name='createWorkflowAsync',
            library='Internal-Workflow',
            service_date='2017_11',
            service_name='Workflow',
            params={'inputData': inputData},
            response_cls=None,
        )



from __future__ import annotations

from tcsoa.gen.GlobalMultiSite._2011_06.ImportExport import CreateGSIdentitiesResponse, GetHashedUIDResponse, TIEGSIdentityInput
from typing import List
from tcsoa.base import TcService


class ImportExportService(TcService):

    @classmethod
    def createGSIdentities(cls, gsIdVect: List[TIEGSIdentityInput]) -> CreateGSIdentitiesResponse:
        """
        This operation takes array of GSIdentityInput structure as input and creates GSIdentity objects in Teamcenter.
        This operation allows for creation of GSIdentities in bulk which is required for GMS co-existence scenarios
        following bulk load import of legacy data into Teamcenter.Whenever an object is exported from a source site the
        record of each imported object is stored in GSIdentity object,which has some basic information of the site that
        owns the object,the type of the class of imported object and 14 digit Unique Identifier string (UID)
        represtening the object.Every imported object will have a corresponding entry in GSIdentity object.This entry
        will be used later during a re-import or sychronize operations internally by importer module.
        
        Use cases:
        This operation is used by user to create GSIds for objects imported by bulk loader.It creates GSIds for non
        GSId based TcXML objects.
        """
        return cls.execute_soa_method(
            method_name='createGSIdentities',
            library='GlobalMultiSite',
            service_date='2011_06',
            service_name='ImportExport',
            params={'gsIdVect': gsIdVect},
            response_cls=CreateGSIdentitiesResponse,
        )

    @classmethod
    def getHashedUID(cls, ownSiteId: int, hashKey: str) -> GetHashedUIDResponse:
        """
        This operation takes a hash key as input and generates a valid Teamcenter Unique Identifier  a 14 character
        long unique string UID based on it. For migrating data from legacy systems to Teamcenter using bulk load import
        of TC XML, this operation can be used to generate UIDs for legacy objects. The UID is composed by using Fowler
        Noll Vo (FNV) hash algorithm for an arbitrary and unique input string.
        
        Use cases:
        This is used during data migration between legacy system such as Enterpise to Teamcenter .
        Used by the importer to generate a UID.
        """
        return cls.execute_soa_method(
            method_name='getHashedUID',
            library='GlobalMultiSite',
            service_date='2011_06',
            service_name='ImportExport',
            params={'ownSiteId': ownSiteId, 'hashKey': hashKey},
            response_cls=GetHashedUIDResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FindWorkspaceObjectsOutput(TcBaseObj):
    """
    FindWorkspaceObjectsOutput
    
    :var findSetIndex: The index of FindSet that generated the output.
    :var foundObjects: The WorkspaceObject business objects matching the search criteria.
    """
    findSetIndex: int = 0
    foundObjects: List[WorkspaceObject] = ()


@dataclass
class FindWorkspaceObjectsResponse(TcBaseObj):
    """
    FindWorkspaceObjectsResponse
    
    :var outputList: A list of found workspace objects.
    :var serviceData: The service data.
    """
    outputList: List[FindWorkspaceObjectsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class WSOFindCriteria(TcBaseObj):
    """
    WSOFindCriteria
    
    :var objectType: If set, must be set to the corresponding class name . If not set, this criteria will be ignored by
    the search.
    :var objectName: Name of object 
    Can contain wild card characters. 
    The percent sign and underscore are the only valid wild card characters, 
    where: 
    % Represents any set of characters in the substring. 
    _ (Underscore) Represents any single character in the substring.
    :var scope: Scope of search,  WSO_scope_All = entire database,  WSO_search_Approved = searches for objects that
    have at least  one status of approval.
    :var owner: The owning User to limit found objects for. A null value will expand the search to all users.
    :var group: The owning Group to limit found objects for. A null value will expand the search to all groups.
    :var createdBefore: The maximum creation date to limit found objects. A null value will not limit the results.
    :var modifiedBefore: The maximum last modified date to limit found objects. A null value will not limit the results.
    :var releasedBefore: The maximum released date to limit found objects. A null value will not limit the results.
    :var createdAfter: The minimum creation date to limit found objects. A null value will not limit the results.
    :var modifiedAfter: The minimum last modified date to limit found objects. A null value will not limit the results.
    :var releasedAfter: The minimum released date to limit found objects. A null value will not limit the results.
    """
    objectType: str = ''
    objectName: str = ''
    scope: WSOscopeType = None
    owner: BusinessObject = None
    group: BusinessObject = None
    createdBefore: datetime = None
    modifiedBefore: datetime = None
    releasedBefore: datetime = None
    createdAfter: datetime = None
    modifiedAfter: datetime = None
    releasedAfter: datetime = None


@dataclass
class WSOFindSet(TcBaseObj):
    """
    WSOFindSet
    
    :var criterias: A list of FindCriteria to search with.
    """
    criterias: List[WSOFindCriteria] = ()


class WSOscopeType(Enum):
    """
    Scope of search:  WSO_scope_All = entire database,  WSO_scope_Approved = searches for objects that have at least
    one approval status.oval status.
    """
    WSO_scope_All = 'WSO_scope_All'
    WSO_scope_Approved = 'WSO_scope_Approved'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, ImanRelation
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LinkBOPtoBOEObjectInfo(TcBaseObj):
    """
    A list of LinkBOPtoBOEObjectInfo structures is the input to the SOA linkPlantBOPtoBOE. This structure has
    1. sourceLine
    2. targetLine
    3. clientId
    
    :var sourceLine: BOP line as  primary object.
    :var targetLine: BOE line as secondary object.
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this LinkBOPtoBOEObjectInfo structure.
    """
    sourceLine: BOMLine = None
    targetLine: BOMLine = None
    clientID: str = ''


@dataclass
class LinkPlantBOPtoBOEResponse(TcBaseObj):
    """
    This structure represents the response from the SOA linkPlantBOPtoBOE for a list of input LinkBOPtoBOEObjectInfo
    structures.
     It has
    1. relationObjects
    2. serviceData
    
    :var relationObjects: A list of created relations.
    :var serviceData: Partial errors as part of the serviceData.
    """
    relationObjects: List[LinkedRelationObject] = ()
    serviceData: ServiceData = None


@dataclass
class LinkedRelationObject(TcBaseObj):
    """
    This structure represents the output from the SOA linkPlantBOPtoBOE  for a single input LinkBOPtoBOEObjectInfo
    structure.
    It has
    1. clientId
    2. relation
    
    :var clientId: The unmodified value from the LinkBOPtoBOEObjectInfo.clientId. This can be used by the caller to
    identify this data structure with the source input data.
    :var relation: The newly created relation of type Mfg0MELinkedBOERel.
    """
    clientId: str = ''
    relation: ImanRelation = None


@dataclass
class SynchronizeBOPAndBOEInputInfo(TcBaseObj):
    """
    A list of SynchronizeBOPAndBOEInputInfo structures is the input to the SOA synchronizePlantBOPAndBOE. This
    structure has
    1. sourceLine
    2. targetLine
    3. removeObsoleteTwin
    4. clientId
    
    :var sourceLine: The source line can either be a plant BOP line or a BOE line. This is the primary structure in the
    synchronization,  based on which the target structure will be modified and its attributes will be mapped.
    :var targetLine: The target line can either be a BOE line or a plant BOP line. The target structure will be
    modified in order to have the similar structure as that of source.
    :var removeObsoleteTwin: A boolean variable to specify if an obsolete object in target structure are to be removed.
    If true, each line in the target structure which do not have any connected line with Mfg0MELinkedBOERel relation
    will be removed. If false, such obolete lines will be unchanged.
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this SynchronizeBOPAndBOEInputInfo structure.
    """
    sourceLine: BOMLine = None
    targetLine: BOMLine = None
    removeObsoleteTwin: bool = False
    clientID: str = ''


@dataclass
class SynchronizePlantBOPAndBOEOutput(TcBaseObj):
    """
    This structure represents the output from the SOA synchronizePlantBOPAndBOE for a single input
    SynchronizeBOPAndBOEInputInfo structure.
    It has
    1. clientId
    2. relations
    
    :var clientId: The unmodified value from the         SynchronizeBOPAndBOEInputInfo.clientId. This can be used by
    the caller to identify this data structure with the source input data.
    :var relations: A list of newly created relations of type Mfg0MELinkedBOERel.
    """
    clientId: str = ''
    relations: List[ImanRelation] = ()


@dataclass
class SynchronizePlantBOPAndBOEResponse(TcBaseObj):
    """
    This structure represents the response from the SOA synchronizePlantBOPAndBOE for a list of input
    SynchronizeBOPAndBOEInputInfo structures.
    It has
    1.synchronizePlantBOPAndBOEOutput: The list of new relations created
    2. serviceData: Partial errors as part of the serviceData.
    
    :var synchronizePlantBOPAndBOEOutput: The list of new relations created.
    :var serviceData: Partial errors as part of the serviceData.
    """
    synchronizePlantBOPAndBOEOutput: List[SynchronizePlantBOPAndBOEOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateBOEfromPlantBOPResponse(TcBaseObj):
    """
    Response from SOA createBOEfromPlantBOP.
    1. boeStructureTopLines
        Top lines of new created BOE structures.
    2. serviceData:
        Partial errors as part of the serviceData.
    
    :var boeStructureTopLines: Top lines of new created BOE structures.
    :var serviceData: Partial errors as part of the serviceData.
    """
    boeStructureTopLines: List[BOMLine] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject, ReportDefinition
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class BOMLineNetEffectivityDetail(TcBaseObj):
    """
    structure to encapsulate a BOMLine and the associated endItem effectivity details.
    
    :var line: The bomline for which the effectivity details will be added.
    :var effDetails: The array of effectivity details for the bomline.
    """
    line: BOMLine = None
    effDetails: List[EndItemDetail] = ()


@dataclass
class ReportCriteria(TcBaseObj):
    """
    criteria for generating print report
    
    :var rdTag: report definition Tag (required)
    :var reportName: name of the report.
    :var datasetType: Dataset type to be used.
    :var reportOptionsNames: StructElement name="reportOptionsNames" description="a vector of strings containing the
    Names in a series of Name/Value pairs used to specify additional criteria (optional)
    :var reportOptionsValues: a vector of strings containing the Values in a series of Name/Value pairs used to specify
    additional criteria (optional)
    :var contextObjects: a vector of Tags representing context object(s) (required for item reports)
    :var contextObjectUIDs: a vector of uids representing context objects
    :var stylesheetTag: stylesheet Tag (optional)
    :var stylesheetName: Name of the stylesheet.
    :var datasetName: name of containing DataSet (optional)
    :var criteriaName: a vector of strings containing the Names in a series of Name/Value pairs used to specify
    additional criteria (optional)
    :var criteriaValues: a vector of strings containing the Values in a series of Name/Value pairs used to specify
    additional criteria (optional)
    :var datasetCtxUID: The uid for the context dataset
    :var datasetCtxObj: Dataset context tag
    :var relationName: relation name to be used.
    """
    rdTag: ReportDefinition = None
    reportName: str = ''
    datasetType: str = ''
    reportOptionsNames: List[str] = ()
    reportOptionsValues: List[str] = ()
    contextObjects: List[BusinessObject] = ()
    contextObjectUIDs: List[str] = ()
    stylesheetTag: BusinessObject = None
    stylesheetName: str = ''
    datasetName: str = ''
    criteriaName: List[str] = ()
    criteriaValues: List[str] = ()
    datasetCtxUID: str = ''
    datasetCtxObj: BusinessObject = None
    relationName: str = ''


@dataclass
class ACInput(TcBaseObj):
    """
    Provides a set of input values for the accountabilityCheck operation
    
    :var sourceObjects: The source bom lines
    :var targetObjects: The target bom lines
    :var resultName: Name of the OccurrenceGroup to be created.
    :var resultDesc: optional description of the OccurrenceGroup to be created - if OccurrenceGroup report option is
    chosen.
    :var reportCriteria: the criteria for printable report.
    :var reportMode: Indicates in what mode the accountability check is running. It can be occurrence group mode, excel
    report mode, coloring mode, or a combination of report and coloring. Valid values for        reportMode are: 1 -
    coloring mode, 2 - excel report mode, 3 - coloring and excel report mode, 4 - occurrence group mode
    :var options: Sum of integer values representing different UI options
    :var sourceContextLine: The possible source context line
    :var targetContextLine: The possible target context line
    :var matchType: Represents user choice in color display
    :var sourceFilteringRule: The source filtering rule
    :var targetFilteringRule: The target filtering rule
    :var sourceDepth: The source structure search depth; -1 represents all depths
    :var targetDepth: The target structure search depth; -1 represents all depths
    """
    sourceObjects: List[BusinessObject] = ()
    targetObjects: List[BusinessObject] = ()
    resultName: str = ''
    resultDesc: str = ''
    reportCriteria: ReportCriteria = None
    reportMode: int = 0
    options: int = 0
    sourceContextLine: BusinessObject = None
    targetContextLine: BusinessObject = None
    matchType: int = 0
    sourceFilteringRule: str = ''
    targetFilteringRule: str = ''
    sourceDepth: int = 0
    targetDepth: int = 0


@dataclass
class TransientFileInfo(TcBaseObj):
    """
    info about any transient report files that are generated.
    
    :var fileName: Name of the file generated in transient volume.
    :var isText: Flag to indicate whether the file is a text file or binary file.
    :var ticket: transient file ticket string.
    """
    fileName: str = ''
    isText: bool = False
    ticket: str = ''


@dataclass
class UnitAndLineDetails(TcBaseObj):
    """
    Structure to capture the Equivalent BomLines (potentially from 2 windows - a source and target window) along with
    their matched units or dates.
    
    :var units: all or a subset of Unit numbers/dates associated with the source and targetlines. The units are in
    pairs - meaning - if you have unit effectivtiy like:1-7,10 - this array will be 1,7,10,10.
    :var dates: If date effectivity is  used - this will have the date ranges for the equivalent lines. Current
    implementation does not support this.
    :var srcLines: list of of srclines that satisfy the units/dates in this structure.
    :var targetLines: list of of target lines ( lines from another window  that are equivalent in some way to the src
    lines - eg: ID in context) that satisfy the units/dates in this structure.
    """
    units: List[int] = ()
    dates: List[datetime] = ()
    srcLines: List[BOMLine] = ()
    targetLines: List[BOMLine] = ()


@dataclass
class AccountabilityCheckResponse(TcBaseObj):
    """
    Contains all the results from the accountabilityCheck operation
    
    :var accountabilityCheckResults: A vector of accountability check results
    :var reachableTargets: A vector of reachable target lines
    :var serviceData: The service data
    """
    accountabilityCheckResults: List[AccountabilityCheckResult] = ()
    reachableTargets: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class AccountabilityCheckResult(TcBaseObj):
    """
    Encapsulates one accountability check result
    
    :var sourceLine: The source bom line
    :var equivalentSourceLines: A vector of equivalent source lines
    :var equivalentTargetLines: A vector of equivalent target lines
    :var checkResult: Accountability check result represented by a color value
    :var resultViewTag: If OccurrenceGroup report option is chosen, this will be the created OccurrenceGroup.
    :var reportFileInfo: details of report files generated in the transient volume.
    """
    sourceLine: BusinessObject = None
    equivalentSourceLines: List[BusinessObject] = ()
    equivalentTargetLines: List[BusinessObject] = ()
    checkResult: int = 0
    resultViewTag: BusinessObject = None
    reportFileInfo: List[TransientFileInfo] = ()


@dataclass
class CompareNetEffectivityGroup(TcBaseObj):
    """
    structure to capture the response of CompareNetEffectivity method.
    
    :var srcLineEffectivities: effectivities of the source lines.
    :var targetLineEffectivities: effectivities of targetLines.
    :var missingSrcDetails: details of missing src by effectivity comparison.
    :var missingTargetDetails: Details of missing target effectivities.
    :var overlappingEffectivities: Details of overlapping effectivities. Source lines that have overlapping units or
    target lines that have overlapping units will be listed.
    :var matchingEffectivities: Details of matching source and target effectivities
    :var isMisMatch: flag that is set to true if there is a mismatch.
    """
    srcLineEffectivities: List[BOMLineNetEffectivityDetail] = ()
    targetLineEffectivities: List[BOMLineNetEffectivityDetail] = ()
    missingSrcDetails: List[EndItemAndUnitDetails] = ()
    missingTargetDetails: List[EndItemAndUnitDetails] = ()
    overlappingEffectivities: List[EndItemAndUnitDetails] = ()
    matchingEffectivities: List[EndItemAndUnitDetails] = ()
    isMisMatch: bool = False


@dataclass
class CompareNetEffectivityResponse(TcBaseObj):
    """
    structure to capture the response of compareNetEffectivityGroup method.  Vector of CompareNetEffectivityGroup
    structures one per input set based equivalent lines and a serviceData member to report partial errors.
    
    :var compareGroups: vector of the compare results for each corresponding set of input vector of equivalent lines.
    :var serviceData: serviceData to return any partial errors.
    """
    compareGroups: List[CompareNetEffectivityGroup] = ()
    serviceData: ServiceData = None


@dataclass
class EndItemAndUnitDetails(TcBaseObj):
    """
    endItem (item used to specify the unit effectivity) tag and the associated UnitAndLineDetails structure.
    
    :var endItem: The endItem object to be associated with the Unit details.
    :var details: structure capturing the details of line(s) and effectivities.
    """
    endItem: BusinessObject = None
    details: List[UnitAndLineDetails] = ()


@dataclass
class EndItemDetail(TcBaseObj):
    """
    structure to capture the details of enditem - the identifier, effectivity units associated with that enditem and
    dates (future).
    
    :var endItem: the endItem to be associated with the units.
    :var units: unit ranges for the endItem. The units are in pairs - meaning - if you have unit effectivtiy
    like:1-7,10 - this array will be 1,7,10,10.
    :var dates: date ranges. Currently not supported. Meaning - will be empty.
    """
    endItem: BusinessObject = None
    units: List[int] = ()
    dates: List[datetime] = ()


@dataclass
class EquivalentLines(TcBaseObj):
    """
    Lines from a Source Window and a Target Window that are equivalent. For example - having the same ID in Context or
    other criteria.
    
    :var eqvSrcLines: set of source bomlines that are equivalent based on some criteria like ID in context.
    :var eqvTargetLines: set of target bomlines (not the same window as source lines) that are equivalent in a manner
    consistent with the source lines.
    """
    eqvSrcLines: List[BOMLine] = ()
    eqvTargetLines: List[BOMLine] = ()
